"""
Settings API models for the Binalyze AIR SDK.
"""

from typing import Optional, Dict, Any, List
from datetime import datetime
from enum import Enum
from pydantic import Field

from ..base import AIRBaseModel


class BannerType(str, Enum):
    """Banner types."""
    INFO = "info"
    WARNING = "warning"
    ERROR = "error"
    SUCCESS = "success"
    MAINTENANCE = "maintenance"


class BannerPosition(str, Enum):
    """Banner display positions."""
    TOP = "top"
    BOTTOM = "bottom"
    CENTER = "center"


class ProxyCertType(str, Enum):
    """Proxy certificate types."""
    PEM = "PEM"
    DER = "DER"
    PKCS12 = "PKCS12"


class ProxySettings(AIRBaseModel):
    """Proxy settings model."""
    enabled: bool = False
    address: Optional[str] = None
    port: Optional[int] = None
    username: Optional[str] = None
    password: Optional[str] = None
    cert_type: Optional[ProxyCertType] = Field(None, alias="certType")
    cert_password: Optional[str] = Field(None, alias="certPassword")
    cert: Optional[str] = None  # Certificate file content


class UpdateProxySettingsRequest(AIRBaseModel):
    """Request model for updating proxy settings."""
    enabled: bool
    address: Optional[str] = None
    port: Optional[int] = None
    username: Optional[str] = None
    password: Optional[str] = None
    cert_type: Optional[ProxyCertType] = Field(None, alias="certType")
    cert_password: Optional[str] = Field(None, alias="certPassword")
    cert: Optional[bytes] = None  # Certificate file data


class ValidateProxySettingsRequest(AIRBaseModel):
    """Request model for validating proxy settings."""
    enabled: bool
    address: Optional[str] = None
    port: Optional[int] = None
    username: Optional[str] = None
    password: Optional[str] = None
    cert_type: Optional[ProxyCertType] = Field(None, alias="certType")
    cert_password: Optional[str] = Field(None, alias="certPassword")
    cert: Optional[bytes] = None  # Certificate file data


class SSLCertificateInfo(AIRBaseModel):
    """SSL certificate information."""
    valid_from: Optional[datetime] = Field(None, alias="validFrom")
    valid_to: Optional[datetime] = Field(None, alias="validTo")
    issuer: Optional[Dict[str, str]] = None
    subject: Optional[Dict[str, str]] = None


class SSLSettings(AIRBaseModel):
    """SSL settings model."""
    ca: Optional[SSLCertificateInfo] = None
    certificate: Optional[SSLCertificateInfo] = None


class SMTPSettings(AIRBaseModel):
    """SMTP settings model."""
    enabled: bool = False
    use_secure_connection: Optional[bool] = Field(None, alias="useSecureConnection")
    server: Optional[str] = None
    port: Optional[int] = None
    username: Optional[str] = None
    from_email: Optional[str] = Field(None, alias="fromEmail")


class SSOProvider(AIRBaseModel):
    """Single Sign-On provider settings."""
    enabled: bool = False
    callback_url: Optional[str] = Field(None, alias="callbackURL")


class OktaSSO(SSOProvider):
    """Okta SSO settings."""
    pass


class AzureSSO(SSOProvider):
    """Azure SSO settings."""
    client_id: Optional[str] = Field(None, alias="clientId")
    tenant_id: Optional[str] = Field(None, alias="tenantId")
    client_secret: Optional[str] = Field(None, alias="clientSecret")


class SingleSignOnSettings(AIRBaseModel):
    """Single Sign-On settings."""
    okta: Optional[OktaSSO] = None
    azure: Optional[AzureSSO] = None


class BackupSchedule(AIRBaseModel):
    """Backup schedule settings."""
    enabled: bool = False
    keep_last: Optional[int] = Field(None, alias="keepLast")
    start_date: Optional[datetime] = Field(None, alias="startDate")
    recurrence: Optional[str] = None


class BackupRestoreSettings(AIRBaseModel):
    """Backup and restore settings."""
    location: Optional[str] = None
    schedule: Optional[BackupSchedule] = None
    encryption_enabled: Optional[bool] = Field(None, alias="encryptionEnabled")


class AutoAssetTaggingSettings(AIRBaseModel):
    """Auto asset tagging settings."""
    enabled: bool = False


class IPRestrictionSettings(AIRBaseModel):
    """IP restriction settings."""
    rules: List[Dict[str, Any]] = []


class LocardSettings(AIRBaseModel):
    """Locard settings."""
    organization: Optional[str] = None
    host: Optional[str] = None
    username: Optional[str] = None
    enabled: bool = False


class PolicySettings(AIRBaseModel):
    """Policy settings."""
    enabled: bool = False


class AgentUpdateSettings(AIRBaseModel):
    """Agent update settings."""
    time_frame: Optional[bool] = Field(None, alias="timeFrame")
    auto_update: Optional[bool] = Field(None, alias="autoUpdate")
    update_from_cdn: Optional[bool] = Field(None, alias="updateFromCdn")


class AgentSettings(AIRBaseModel):
    """Agent settings."""
    tamper_detection_enabled: Optional[bool] = Field(None, alias="tamperDetectionEnabled")
    resolve_public_ip_enabled: Optional[bool] = Field(None, alias="resolvePublicIpEnabled")
    update: Optional[AgentUpdateSettings] = None
    uninstallation_password_protection_enabled: Optional[bool] = Field(None, alias="uninstallationPasswordProtectionEnabled")


class InterACTSettings(AIRBaseModel):
    """InterACT settings."""
    enabled: bool = False


class RFC3161Settings(AIRBaseModel):
    """RFC3161 settings."""
    enabled: bool = False


class MitreAttackSettings(AIRBaseModel):
    """MITRE ATT&CK settings."""
    version: Optional[str] = None
    last_sync: Optional[datetime] = Field(None, alias="lastSync")


class FleetAISettings(AIRBaseModel):
    """Fleet AI settings."""
    enabled: bool = False


class AutoUpdateSettings(AIRBaseModel):
    """Auto update settings."""
    update_time_type: Optional[str] = Field(None, alias="updateTimeType")
    update_time: Optional[str] = Field(None, alias="updateTime")
    update_day: Optional[str] = Field(None, alias="updateDay")
    enabled: bool = False


class ActiveDirectorySettings(AIRBaseModel):
    """Active Directory settings model."""
    enabled: bool = False
    use_secure_connection: Optional[bool] = Field(None, alias="useSecureConnection")
    query: Optional[str] = None
    server: Optional[str] = None
    domain: Optional[str] = None
    username: Optional[str] = None
    password: Optional[str] = None


class AllSettings(AIRBaseModel):
    """Complete settings model representing all system settings."""
    
    # Core settings
    language: Optional[str] = None
    log_level: Optional[str] = Field(None, alias="logLevel")
    automatic_error_reporting: Optional[bool] = Field(None, alias="automaticErrorReporting")
    multi_port_enabled: Optional[bool] = Field(None, alias="multiPortEnabled")
    enforce_tfa_globally: Optional[bool] = Field(None, alias="enforceTFAGlobally")
    console_address: Optional[str] = Field(None, alias="consoleAddress")
    console_id: Optional[str] = Field(None, alias="consoleId")
    mandatory_case_selection_enabled: Optional[bool] = Field(None, alias="mandatoryCaseSelectionEnabled")
    statistics_start_date: Optional[datetime] = Field(None, alias="statisticsStartDate")
    
    # Complex settings objects
    ssl: Optional[SSLSettings] = None
    proxy: Optional[ProxySettings] = None
    smtp: Optional[SMTPSettings] = None
    active_directory: Optional[ActiveDirectorySettings] = Field(None, alias="activeDirectory")
    syslog: Optional[Dict[str, Any]] = None
    single_sign_on: Optional[SingleSignOnSettings] = Field(None, alias="singleSignOn")
    backup_restore: Optional[BackupRestoreSettings] = Field(None, alias="backupRestore")
    auto_asset_tagging: Optional[AutoAssetTaggingSettings] = Field(None, alias="autoAssetTagging")
    ip_restriction: Optional[IPRestrictionSettings] = Field(None, alias="ipRestriction")
    locard: Optional[LocardSettings] = None
    banner: Optional[Dict[str, Any]] = None
    policy: Optional[PolicySettings] = None
    agent: Optional[AgentSettings] = None
    interact: Optional[InterACTSettings] = Field(None, alias="interACT")
    rfc3161: Optional[RFC3161Settings] = None
    mitre_attack: Optional[MitreAttackSettings] = Field(None, alias="mitreAttack")
    feature_flags: Optional[Dict[str, Any]] = Field(None, alias="featureFlags")
    fleet_ai: Optional[FleetAISettings] = Field(None, alias="fleetAi")
    auto_update: Optional[AutoUpdateSettings] = Field(None, alias="autoUpdate")
    
    # Metadata
    created_at: Optional[datetime] = Field(None, alias="createdAt")
    updated_at: Optional[datetime] = Field(None, alias="updatedAt")
    id: Optional[str] = Field(None, alias="_id")


class GeneralSettingsRequest(AIRBaseModel):
    """Request model for updating general settings."""
    
    log_level: Optional[str] = Field(None, alias="logLevel")
    multi_port_enabled: Optional[bool] = Field(None, alias="multiPortEnabled")
    mandatory_case_selection_enabled: Optional[bool] = Field(None, alias="mandatoryCaseSelectionEnabled")
    automatic_error_reporting: Optional[bool] = Field(None, alias="automaticErrorReporting")
    enforce_tfa_globally: Optional[bool] = Field(None, alias="enforceTFAGlobally")
    console_address: Optional[str] = Field(None, alias="consoleAddress")


class BannerSettings(AIRBaseModel):
    """Banner settings model."""
    
    id: Optional[str] = None
    enabled: bool = False
    title: Optional[str] = None
    message: str
    banner_type: BannerType = BannerType.INFO
    position: BannerPosition = BannerPosition.TOP
    dismissible: bool = True
    auto_dismiss: bool = False
    auto_dismiss_timeout: Optional[int] = None  # seconds
    show_from: Optional[datetime] = None
    show_until: Optional[datetime] = None
    background_color: Optional[str] = None
    text_color: Optional[str] = None
    border_color: Optional[str] = None
    icon: Optional[str] = None
    link_url: Optional[str] = None
    link_text: Optional[str] = None
    target_roles: Optional[list[str]] = None
    target_organizations: Optional[list[int]] = None
    created_by: Optional[str] = None
    created_at: Optional[datetime] = None
    updated_at: Optional[datetime] = None
    organization_id: Optional[int] = None


class UpdateBannerSettingsRequest(AIRBaseModel):
    """Request model for updating banner settings with proper API field mapping."""
    
    enabled: Optional[bool] = None
    title: Optional[str] = None
    message: Optional[str] = None
    # API expects these exact field names - use aliases to map from Python names to API names
    users_can_dismiss: Optional[bool] = Field(default=None, alias="usersCanDismiss")
    color: Optional[str] = None  # API expects: general, info, maintenance, warning, alert
    display_time_type: Optional[str] = Field(default=None, alias="displayTimeType")  # always or scheduled
    schedule_times: Optional[Dict[str, Any]] = Field(default=None, alias="scheduleTimes")
    
    # Legacy/additional fields (may not be used by current API)
    banner_type: Optional[BannerType] = None
    position: Optional[BannerPosition] = None
    dismissible: Optional[bool] = None
    auto_dismiss: Optional[bool] = None
    auto_dismiss_timeout: Optional[int] = None
    show_from: Optional[datetime] = None
    show_until: Optional[datetime] = None
    background_color: Optional[str] = None
    text_color: Optional[str] = None
    border_color: Optional[str] = None
    icon: Optional[str] = None
    link_url: Optional[str] = None
    link_text: Optional[str] = None
    target_roles: Optional[list[str]] = None
    target_organizations: Optional[list[int]] = None


class UpdateActiveDirectorySettingsRequest(AIRBaseModel):
    """Request model for updating Active Directory settings."""
    enabled: bool
    use_secure_connection: Optional[bool] = Field(None, alias="useSecureConnection")
    query: Optional[str] = None
    server: Optional[str] = None
    domain: Optional[str] = None
    username: Optional[str] = None
    password: Optional[str] = None


class ValidateActiveDirectorySettingsRequest(AIRBaseModel):
    """Request model for validating Active Directory settings."""
    server: str
    domain: str
    username: str
    password: str
    use_secure_connection: Optional[bool] = Field(None, alias="useSecureConnection")


class UpdateSMTPSettingsRequest(AIRBaseModel):
    """Request model for updating SMTP settings."""
    enabled: bool
    server: Optional[str] = None
    port: Optional[int] = None
    use_secure_connection: Optional[bool] = Field(None, alias="useSecureConnection")
    username: Optional[str] = None
    password: Optional[str] = None
    to_address: Optional[str] = Field(None, alias="toAddress")
    from_email: Optional[str] = Field(None, alias="fromEmail")


class ValidateSMTPSettingsRequest(AIRBaseModel):
    """Request model for validating SMTP settings."""
    enabled: bool
    server: Optional[str] = None
    port: Optional[int] = None
    use_secure_connection: Optional[bool] = Field(None, alias="useSecureConnection")
    username: Optional[str] = None
    password: Optional[str] = None
    to_address: Optional[str] = Field(None, alias="toAddress")
    from_email: Optional[str] = Field(None, alias="fromEmail")


class UpdateConsoleAddressSettingsRequest(AIRBaseModel):
    """Request model for updating console address settings."""
    console_address: str = Field(alias="consoleAddress")
    cert_type: Optional[str] = Field(None, alias="certType")  # PEM, DER, PKCS12
    cert_password: Optional[str] = Field(None, alias="certPassword")
    cert: Optional[bytes] = None  # Certificate file data
    key: Optional[bytes] = None  # Key file data


class ValidateConsoleAddressSettingsRequest(AIRBaseModel):
    """Request model for validating console address settings."""
    console_address: str = Field(alias="consoleAddress")


class UpdatePolicySettingsRequest(AIRBaseModel):
    """Request model for updating policy settings."""
    enabled: bool


class UpdateEnforceTFASettingsRequest(AIRBaseModel):
    """Request model for updating enforce TFA settings."""
    enabled: bool


class UpdateAutoAssetTaggingSettingsRequest(AIRBaseModel):
    """Request model for updating auto asset tagging settings."""
    enabled: bool


class UpdateInteractSettingsRequest(AIRBaseModel):
    """Request model for updating InterACT settings."""
    enabled: bool


class UpdateRFC3161SettingsRequest(AIRBaseModel):
    """Request model for updating RFC3161 settings."""
    enabled: bool


class UpdateResponderSettingsRequest(AIRBaseModel):
    """Request model for updating responder/agent settings."""
    enabled: bool


class UpdateLocardSettingsRequest(AIRBaseModel):
    """Request model for updating Locard settings."""
    enabled: bool
    organization: Optional[str] = None
    host: Optional[str] = None
    username: Optional[str] = None


class UpdateUninstallationPasswordProtectionSettingsRequest(AIRBaseModel):
    """Request model for updating uninstallation password protection settings."""
    enabled: bool = Field(alias="enabled")


class UpdateSyslogSettingsRequest(AIRBaseModel):
    """Request model for updating syslog settings"""
    enabled: bool = Field(alias="enabled")
    server: str = Field(alias="server")
    port: int = Field(alias="port")
    protocol: str = Field(alias="protocol")  # tcp or udp


class ValidateSyslogSettingsRequest(AIRBaseModel):
    """Request model for validating syslog settings"""
    enabled: bool = Field(alias="enabled")
    server: str = Field(alias="server")
    port: int = Field(alias="port")
    protocol: str = Field(alias="protocol")  # tcp or udp


class LocalBackupSettings(AIRBaseModel):
    """Local backup settings model"""
    path: str = Field(alias="path")


class SftpBackupSettings(AIRBaseModel):
    """SFTP backup settings model"""
    host: str = Field(alias="host")
    port: int = Field(alias="port", default=22)
    logon_type: str = Field(alias="logonType")  # normal or keyfile
    username: str = Field(alias="username")
    password: str = Field(alias="password", default="")
    private_key: str = Field(alias="privateKey", default="")
    passphrase: str = Field(alias="passphrase", default="")


class BackupScheduleSettings(AIRBaseModel):
    """Backup schedule settings model"""
    enabled: bool = Field(alias="enabled")
    recurrence: str = Field(alias="recurrence", default="monthly")  # daily, weekly, monthly
    start_date: str = Field(alias="startDate")  # ISO datetime
    keep_last: int = Field(alias="keepLast", default=5)


class UpdateBackupRestoreSettingsRequest(AIRBaseModel):
    """Request model for updating backup restore settings"""
    location: str = Field(alias="location")  # local or sftp
    encryption_enabled: bool = Field(alias="encryptionEnabled", default=False)
    encryption_password: str = Field(alias="encryptionPassword", default="")
    local: LocalBackupSettings = Field(alias="local")
    sftp: SftpBackupSettings = Field(alias="sftp")
    schedule: BackupScheduleSettings = Field(alias="schedule")


class ValidateBackupSftpSettingsRequest(AIRBaseModel):
    """Request model for validating backup SFTP settings"""
    host: str = Field(alias="host")
    port: int = Field(alias="port", default=22)
    path: str = Field(alias="path", default="")
    logon_type: str = Field(alias="logonType")  # normal or keyfile
    username: str = Field(alias="username")
    password: str = Field(alias="password", default="")
    private_key: str = Field(alias="privateKey", default="")
    passphrase: str = Field(alias="passphrase", default="")


class AzureSsoSettings(AIRBaseModel):
    """Azure SSO settings model"""
    enabled: bool = Field(alias="enabled")
    tenant_id: str = Field(alias="tenantId")
    client_id: str = Field(alias="clientId")
    client_secret: str = Field(alias="clientSecret")


class OktaSsoSettings(AIRBaseModel):
    """Okta SSO settings model"""
    enabled: bool = Field(alias="enabled")
    entry_point: str = Field(alias="entryPoint")
    issuer: str = Field(alias="issuer")
    cert: str = Field(alias="cert")


class UpdateSingleSignOnSettingsRequest(AIRBaseModel):
    """Request model for updating single sign-on settings"""
    azure: AzureSsoSettings = Field(alias="azure")
    okta: OktaSsoSettings = Field(alias="okta")


class UpdateSslSettingsRequest(AIRBaseModel):
    """Request model for updating SSL settings (multipart form data)"""
    cert_type: str = Field(alias="certType", default="PEM")  # PEM or PKCS12
    cert_password: str = Field(alias="certPassword", default="")
    # Note: cert and key files handled separately in multipart upload


class ValidateSslSettingsRequest(AIRBaseModel):
    """Request model for validating SSL settings (multipart form data)"""
    cert_type: str = Field(alias="certType", default="PEM")  # PEM or PKCS12
    cert_password: str = Field(alias="certPassword", default="")
    # Note: cert and key files handled separately in multipart upload


class SslCertificateSubject(AIRBaseModel):
    """SSL certificate subject information"""
    country_name: str = Field(alias="countryName")
    state_or_province_name: str = Field(alias="stateOrProvinceName")
    locality_name: str = Field(alias="localityName", default="")
    organization_name: str = Field(alias="organizationName")
    organizational_unit_name: str = Field(alias="organizationalUnitName", default="")
    common_name: str = Field(alias="commonName")
    email_address: str = Field(alias="emailAddress", default="")


class SslCertificateValidation(AIRBaseModel):
    """SSL certificate validation response"""
    issuer: SslCertificateSubject = Field(alias="issuer")
    subject: SslCertificateSubject = Field(alias="subject")
    valid_from: str = Field(alias="validFrom")  # ISO datetime
    valid_to: str = Field(alias="validTo")  # ISO datetime 


"""
Investigation Hub API for the Binalyze AIR SDK.
"""

from typing import Optional, List, Dict, Any

from ..http_client import HTTPClient
from ..models.investigation_hub import (
    Investigation, InvestigationAsset, FlagSummary, EvidenceSection,
    EvidenceStructure, SQLQueryResult, FindingsSummary, FindingsStructure,
    FindingsResult, FindingsRequest, MitreMatch, InvestigationComment, 
    InvestigationActivity, AdvancedFilter, UpdateInvestigationRequest, 
    FlagEvidenceRequest, AddNoteRequest, CreateCommentRequest, 
    MarkActivityAsReadRequest, CreateAdvancedFilterRequest,
    UpdateAdvancedFilterRequest, ExportRequest
)
from ..queries.investigation_hub import (
    GetInvestigationQuery, GetInvestigationAssetsQuery, GetInvestigationFlagSummaryQuery,
    GetEvidenceSectionsQuery, GetEvidenceStructureQuery, ExecuteSQLQuery,
    GetFindingsSummaryQuery, GetFindingsStructureQuery, GetFindingsQuery,
    GetMitreMatchesQuery, GetInvestigationCommentsQuery,
    GetInvestigationActivitiesQuery, GetAdvancedFiltersQuery, GetAdvancedFilterQuery,
    GetEvidenceRecordsQuery
)
from ..commands.investigation_hub import (
    UpdateInvestigationCommand, DeleteInvestigationCommand, FlagEvidenceCommand,
    UnflagEvidenceCommand, AddNoteToEvidenceCommand, CreateCommentCommand,
    UpdateCommentCommand, DeleteCommentCommand, MarkActivitiesAsReadCommand,
    CreateAdvancedFilterCommand, UpdateAdvancedFilterCommand, DeleteAdvancedFilterCommand,
    ExportEvidenceCommand, GenerateReportCommand, RefreshInvestigationCommand,
    ArchiveInvestigationCommand, RestoreInvestigationCommand
)


class InvestigationHubAPI:
    """Investigation Hub API with CQRS pattern - separated queries and commands."""
    
    def __init__(self, http_client: HTTPClient):
        self.http_client = http_client
    
    # QUERIES (Read operations)
    def get_investigation(self, investigation_id: str) -> Investigation:
        """Get a specific investigation by ID."""
        query = GetInvestigationQuery(self.http_client, investigation_id)
        return query.execute()
    
    def get_assets(self, investigation_id: str) -> List[InvestigationAsset]:
        """Get assets for a specific investigation."""
        query = GetInvestigationAssetsQuery(self.http_client, investigation_id)
        return query.execute()
    
    def get_flag_summary(self, investigation_id: str) -> List[FlagSummary]:
        """Get flag summary for a specific investigation."""
        query = GetInvestigationFlagSummaryQuery(self.http_client, investigation_id)
        return query.execute()
    
    def get_evidence_sections(self, investigation_id: str, task_assignment_ids: List[str]) -> List[EvidenceSection]:
        """Get evidence sections for a specific investigation with task assignment IDs."""
        query = GetEvidenceSectionsQuery(self.http_client, investigation_id, task_assignment_ids)
        return query.execute()
    
    def get_evidence_structure(self, investigation_id: str, section: str) -> List[EvidenceStructure]:
        """Get evidence structure for a specific section."""
        query = GetEvidenceStructureQuery(self.http_client, investigation_id, section)
        return query.execute()
    
    def execute_sql_query(self, investigation_id: str, query: str, 
                         page_size: int = 10, page_number: int = 1) -> SQLQueryResult:
        """Execute SQL query against investigation database."""
        sql_query = ExecuteSQLQuery(self.http_client, investigation_id, query, page_size, page_number)
        return sql_query.execute()
    
    def get_findings_summary(self, investigation_id: str) -> FindingsSummary:
        """Get findings summary for a specific investigation."""
        query = GetFindingsSummaryQuery(self.http_client, investigation_id)
        return query.execute()
    
    def get_findings_structure(self, investigation_id: str) -> FindingsStructure:
        """Get findings structure for a specific investigation."""
        query = GetFindingsStructureQuery(self.http_client, investigation_id)
        return query.execute()
    
    def get_findings(self, investigation_id: str, request: Optional[FindingsRequest] = None) -> FindingsResult:
        """Get findings for a specific investigation."""
        if request is None:
            request = FindingsRequest()
        query = GetFindingsQuery(self.http_client, investigation_id, request)
        return query.execute()
    
    def get_mitre_matches(self, investigation_id: str) -> List[MitreMatch]:
        """Get MITRE ATT&CK matches for a specific investigation."""
        query = GetMitreMatchesQuery(self.http_client, investigation_id)
        return query.execute()
    
    def get_comments(self, investigation_id: str, evidence_id: Optional[str] = None) -> List[InvestigationComment]:
        """Get comments for a specific investigation."""
        query = GetInvestigationCommentsQuery(self.http_client, investigation_id, evidence_id)
        return query.execute()
    
    def get_activities(self, investigation_id: str, page_size: int = 20, 
                      page_number: int = 1) -> List[InvestigationActivity]:
        """Get activities for a specific investigation."""
        query = GetInvestigationActivitiesQuery(self.http_client, investigation_id, page_size, page_number)
        return query.execute()
    
    def get_advanced_filters(self, investigation_id: Optional[str] = None) -> List[AdvancedFilter]:
        """Get advanced filters (organization-wide, not investigation-specific)."""
        query = GetAdvancedFiltersQuery(self.http_client, investigation_id)
        return query.execute()
    
    def get_advanced_filter(self, investigation_id: str, filter_id: str) -> AdvancedFilter:
        """Get a specific advanced filter by ID."""
        query = GetAdvancedFilterQuery(self.http_client, investigation_id, int(filter_id))
        return query.execute()
    
    def get_evidence_records(self, investigation_id: str, section: str,
                            filters: Optional[Dict[str, Any]] = None, 
                            page_size: int = 50, page_number: int = 1) -> Dict[str, Any]:
        """Get evidence records with filtering."""
        query = GetEvidenceRecordsQuery(
            self.http_client, investigation_id, section, filters, page_size, page_number
        )
        return query.execute()
    
    # COMMANDS (Write operations)
    def update_investigation(self, investigation_id: str, request: UpdateInvestigationRequest) -> Investigation:
        """Update an investigation."""
        command = UpdateInvestigationCommand(self.http_client, investigation_id, request)
        return command.execute()
    
    def delete_investigation(self, investigation_id: str) -> Dict[str, Any]:
        """Delete an investigation."""
        command = DeleteInvestigationCommand(self.http_client, investigation_id)
        return command.execute()
    
    def flag_evidence(self, investigation_id: str, request: FlagEvidenceRequest) -> Dict[str, Any]:
        """Flag evidence records."""
        command = FlagEvidenceCommand(self.http_client, investigation_id, request)
        return command.execute()
    
    def unflag_evidence(self, investigation_id: str, records: List[Dict[str, Any]], 
                       section: str) -> Dict[str, Any]:
        """Unflag evidence records."""
        command = UnflagEvidenceCommand(self.http_client, investigation_id, records, section)
        return command.execute()
    
    def add_note_to_evidence(self, investigation_id: str, request: AddNoteRequest) -> Dict[str, Any]:
        """Add a note to evidence records."""
        command = AddNoteToEvidenceCommand(self.http_client, investigation_id, request)
        return command.execute()
    
    def create_comment(self, investigation_id: str, request: CreateCommentRequest) -> InvestigationComment:
        """Create a comment in an investigation."""
        command = CreateCommentCommand(self.http_client, investigation_id, request)
        return command.execute()
    
    def update_comment(self, investigation_id: str, comment_id: str, content: str) -> InvestigationComment:
        """Update a comment in an investigation."""
        command = UpdateCommentCommand(self.http_client, investigation_id, comment_id, content)
        return command.execute()
    
    def delete_comment(self, investigation_id: str, comment_id: str) -> Dict[str, Any]:
        """Delete a comment from an investigation."""
        command = DeleteCommentCommand(self.http_client, investigation_id, comment_id)
        return command.execute()
    
    def mark_activities_as_read(self, investigation_id: str, 
                               request: MarkActivityAsReadRequest) -> Dict[str, Any]:
        """Mark activities as read."""
        command = MarkActivitiesAsReadCommand(self.http_client, investigation_id, request)
        return command.execute()
    
    def create_advanced_filter(self, investigation_id: str, 
                              request: CreateAdvancedFilterRequest) -> AdvancedFilter:
        """Create an advanced filter."""
        command = CreateAdvancedFilterCommand(self.http_client, investigation_id, request)
        return command.execute()
    
    def update_advanced_filter(self, investigation_id: str, filter_id: str, 
                              request: UpdateAdvancedFilterRequest) -> AdvancedFilter:
        """Update an advanced filter."""
        command = UpdateAdvancedFilterCommand(self.http_client, investigation_id, int(filter_id), request)
        return command.execute()
    
    def delete_advanced_filter(self, investigation_id: str, filter_id: str) -> Dict[str, Any]:
        """Delete an advanced filter."""
        command = DeleteAdvancedFilterCommand(self.http_client, investigation_id, int(filter_id))
        return command.execute()
    
    def export_evidence(self, investigation_id: str, request: ExportRequest) -> Dict[str, Any]:
        """Export evidence data."""
        command = ExportEvidenceCommand(self.http_client, investigation_id, request)
        return command.execute()
    
    def generate_report(self, investigation_id: str, report_type: str = "comprehensive", 
                       include_evidence: bool = True) -> Dict[str, Any]:
        """Generate an investigation report."""
        command = GenerateReportCommand(self.http_client, investigation_id, report_type, include_evidence)
        return command.execute()
    
    def refresh_investigation(self, investigation_id: str) -> Investigation:
        """Refresh/rebuild an investigation."""
        command = RefreshInvestigationCommand(self.http_client, investigation_id)
        return command.execute()
    
    def archive_investigation(self, investigation_id: str) -> Dict[str, Any]:
        """Archive an investigation."""
        command = ArchiveInvestigationCommand(self.http_client, investigation_id)
        return command.execute()
    
    def restore_investigation(self, investigation_id: str) -> Investigation:
        """Restore an archived investigation."""
        command = RestoreInvestigationCommand(self.http_client, investigation_id)
        return command.execute()
    
    # Convenience methods
    def sql_query(self, investigation_id: str, query: str, page_size: int = 10, 
                 page_number: int = 1) -> SQLQueryResult:
        """Alias for execute_sql_query for convenience."""
        return self.execute_sql_query(investigation_id, query, page_size, page_number)
    
    def get_investigation_data(self, investigation_id: str, task_assignment_ids: Optional[List[str]] = None) -> Dict[str, Any]:
        """Get comprehensive investigation data including assets, flags, and sections."""
        result = {
            "investigation": self.get_investigation(investigation_id),
            "assets": self.get_assets(investigation_id),
            "flag_summary": self.get_flag_summary(investigation_id)
        }
        
        # Only include evidence sections if task assignment IDs are provided
        if task_assignment_ids:
            result["evidence_sections"] = self.get_evidence_sections(investigation_id, task_assignment_ids)
        
        return result 


"""
Relay Server commands for the Binalyze AIR SDK.
"""

from typing import Dict, Any

from ..base import Command
from ..models.relay_server import (
    RebootTaskRequest, ShutdownTaskRequest, LogRetrievalTaskRequest, VersionUpdateTaskRequest,
    UpdateTagsRequest, UpdateLabelRequest, UpdateAddressRequest
)
from ..http_client import HTTPClient


class AssignRebootTaskCommand(Command[Dict[str, Any]]):
    """Command to assign reboot task to relay server."""
    
    def __init__(self, http_client: HTTPClient, relay_server_id: str, task_request: RebootTaskRequest):
        self.http_client = http_client
        self.relay_server_id = relay_server_id
        self.task_request = task_request
    
    def execute(self) -> Dict[str, Any]:
        """Execute the command to assign reboot task."""
        response = self.http_client.post(
            f'/relay-servers/{self.relay_server_id}/tasks/reboot',
            json_data=self.task_request.model_dump(exclude_none=True)
        )
        return response


class AssignShutdownTaskCommand(Command[Dict[str, Any]]):
    """Command to assign shutdown task to relay server."""
    
    def __init__(self, http_client: HTTPClient, relay_server_id: str, task_request: ShutdownTaskRequest):
        self.http_client = http_client
        self.relay_server_id = relay_server_id
        self.task_request = task_request
    
    def execute(self) -> Dict[str, Any]:
        """Execute the command to assign shutdown task."""
        response = self.http_client.post(
            f'/relay-servers/{self.relay_server_id}/tasks/shutdown',
            json_data=self.task_request.model_dump(exclude_none=True)
        )
        return response


class AssignLogRetrievalTaskCommand(Command[Dict[str, Any]]):
    """Command to assign log retrieval task to relay server."""
    
    def __init__(self, http_client: HTTPClient, relay_server_id: str, task_request: LogRetrievalTaskRequest):
        self.http_client = http_client
        self.relay_server_id = relay_server_id
        self.task_request = task_request
    
    def execute(self) -> Dict[str, Any]:
        """Execute the command to assign log retrieval task."""
        response = self.http_client.post(
            f'/relay-servers/{self.relay_server_id}/tasks/log-retrieval',
            json_data=self.task_request.model_dump(exclude_none=True)
        )
        return response


class AssignVersionUpdateTaskCommand(Command[Dict[str, Any]]):
    """Command to assign version update task to relay server."""
    
    def __init__(self, http_client: HTTPClient, relay_server_id: str, task_request: VersionUpdateTaskRequest):
        self.http_client = http_client
        self.relay_server_id = relay_server_id
        self.task_request = task_request
    
    def execute(self) -> Dict[str, Any]:
        """Execute the command to assign version update task."""
        response = self.http_client.post(
            f'/relay-servers/{self.relay_server_id}/tasks/version-update',
            json_data=self.task_request.model_dump(exclude_none=True)
        )
        return response


class DeleteRelayServerCommand(Command[Dict[str, Any]]):
    """Command to delete a relay server."""
    
    def __init__(self, http_client: HTTPClient, server_id: str):
        self.http_client = http_client
        self.server_id = server_id
    
    def execute(self) -> Dict[str, Any]:
        """Execute the command to delete a relay server."""
        response = self.http_client.delete('/relay-servers/remove', params={'id': self.server_id})
        return response


class UpdateTagsCommand(Command[Dict[str, Any]]):
    """Command to update tags for a relay server."""
    
    def __init__(self, http_client: HTTPClient, relay_server_id: str, tags_request: UpdateTagsRequest):
        self.http_client = http_client
        self.relay_server_id = relay_server_id
        self.tags_request = tags_request
    
    def execute(self) -> Dict[str, Any]:
        """Execute the command to update tags."""
        response = self.http_client.patch(
            f'/relay-servers/{self.relay_server_id}/tag',
            json_data=self.tags_request.model_dump(exclude_none=True)
        )
        return response


class UpdateLabelCommand(Command[Dict[str, Any]]):
    """Command to update label for a relay server."""
    
    def __init__(self, http_client: HTTPClient, relay_server_id: str, label_request: UpdateLabelRequest):
        self.http_client = http_client
        self.relay_server_id = relay_server_id
        self.label_request = label_request
    
    def execute(self) -> Dict[str, Any]:
        """Execute the command to update label."""
        response = self.http_client.patch(
            f'/relay-servers/{self.relay_server_id}/label',
            json_data=self.label_request.model_dump(exclude_none=True)
        )
        return response


class UpdateAddressCommand(Command[Dict[str, Any]]):
    """Command to update address for a relay server."""
    
    def __init__(self, http_client: HTTPClient, relay_server_id: str, address_request: UpdateAddressRequest):
        self.http_client = http_client
        self.relay_server_id = relay_server_id
        self.address_request = address_request
    
    def execute(self) -> Dict[str, Any]:
        """Execute the command to update address."""
        response = self.http_client.patch(
            f'/relay-servers/{self.relay_server_id}/address',
            json_data=self.address_request.model_dump(exclude_none=True)
        )
        return response 


"""
Baseline-related commands for the Binalyze AIR SDK.
"""

from typing import Dict, Any

from ..base import Command
from ..http_client import HTTPClient


class AcquireBaselineByFilterCommand(Command[Dict[str, Any]]):
    """Command to acquire baselines by asset filter criteria."""
    
    def __init__(self, http_client: HTTPClient, payload: Dict[str, Any]):
        self.http_client = http_client
        self.payload = payload
    
    def execute(self) -> Dict[str, Any]:
        """Execute the command to acquire baselines by filter."""
        return self.http_client.post("baseline/acquire", json_data=self.payload)


class CompareBaselineByEndpointCommand(Command[Dict[str, Any]]):
    """Command to compare baseline acquisition tasks by endpoint ID."""
    
    def __init__(self, http_client: HTTPClient, payload: Dict[str, Any]):
        self.http_client = http_client
        self.payload = payload
    
    def execute(self) -> Dict[str, Any]:
        """Execute the command to compare baseline tasks."""
        return self.http_client.post("baseline/compare", json_data=self.payload) 


"""
Cases API for the Binalyze AIR SDK using CQRS pattern.
"""

from typing import List, Optional, Dict, Any
from ..http_client import HTTPClient
from ..models.cases import (
    Case, CaseActivity, CaseEndpoint, CaseTask, User, CaseFilter, CaseActivityFilter,
    CreateCaseRequest, UpdateCaseRequest, CaseNote, CaseEndpointFilter, CaseTaskFilter, CaseUserFilter
)
from ..models.assets import AssetFilter
from ..queries.cases import (
    ListCasesQuery,
    GetCaseQuery,
    GetCaseActivitiesQuery,
    GetCaseEndpointsQuery,
    GetCaseTasksQuery,
    GetCaseUsersQuery,
    CheckCaseNameQuery,
)
from ..commands.cases import (
    CreateCaseCommand,
    UpdateCaseCommand,
    CloseCaseCommand,
    OpenCaseCommand,
    ArchiveCaseCommand,
    ChangeCaseOwnerCommand,
    RemoveEndpointsFromCaseCommand,
    RemoveTaskAssignmentFromCaseCommand,
    ImportTaskAssignmentsToCaseCommand,
    AddNoteToCaseCommand,
    UpdateNoteToCaseCommand,
    DeleteNoteToCaseCommand,
    ExportCaseNotesCommand,
    ExportCasesCommand,
    ExportCaseEndpointsCommand,
    ExportCaseActivitiesCommand,
)


class CasesAPI:
    """Cases API with CQRS pattern - separated queries and commands."""

    def __init__(self, http_client: HTTPClient):
        self.http_client = http_client

    # QUERIES (Read operations)
    def list(self, filter_params: Optional[CaseFilter] = None, organization_ids: Optional[List[int]] = None) -> List[Case]:
        """List cases with optional filtering."""
        query = ListCasesQuery(self.http_client, filter_params, organization_ids)
        return query.execute()

    def get(self, case_id: str) -> Case:
        """Get a specific case by ID."""
        query = GetCaseQuery(self.http_client, case_id)
        return query.execute()

    def get_activities(self, case_id: str, filter_params: Optional[CaseActivityFilter] = None) -> List[CaseActivity]:
        """Get activities for a specific case with optional filtering, pagination, and sorting."""
        query = GetCaseActivitiesQuery(self.http_client, case_id, filter_params)
        return query.execute()

    def get_endpoints(
        self, case_id: str, filter_params: Optional[CaseEndpointFilter] = None, organization_ids: Optional[List[int]] = None
    ) -> List[CaseEndpoint]:
        """Get endpoints for a specific case with comprehensive filtering support.

        Args:
            case_id: The case ID to get endpoints for
            filter_params: Optional CaseEndpointFilter with comprehensive filtering options
            organization_ids: Optional list of organization IDs (for backward compatibility)

        Returns:
            List of CaseEndpoint objects

        Note: If both filter_params and organization_ids are provided, organization_ids in filter_params takes precedence.
        """
        # Handle backward compatibility
        if filter_params is None:
            filter_params = CaseEndpointFilter()

        # If organization_ids is provided and not set in filter_params, use it
        if organization_ids is not None and filter_params.organization_ids is None:
            filter_params.organization_ids = organization_ids

        query = GetCaseEndpointsQuery(self.http_client, case_id, filter_params)
        return query.execute()

    def get_tasks(
        self, case_id: str, filter_params: Optional[CaseTaskFilter] = None, organization_ids: Optional[List[int]] = None
    ) -> List[CaseTask]:
        """Get tasks for a specific case with comprehensive filtering support.

        Args:
            case_id: The case ID to get tasks for
            filter_params: Optional CaseTaskFilter with comprehensive filtering options
            organization_ids: Optional list of organization IDs (for backward compatibility)

        Returns:
            List of CaseTask objects

        Note: If both filter_params and organization_ids are provided, organization_ids in filter_params takes precedence.
        """
        # Handle backward compatibility
        if filter_params is None:
            filter_params = CaseTaskFilter()

        # If organization_ids is provided and not set in filter_params, use it
        if organization_ids is not None and filter_params.organization_ids is None:
            filter_params.organization_ids = organization_ids

        query = GetCaseTasksQuery(self.http_client, case_id, filter_params)
        return query.execute()

    def get_users(
        self, case_id: str, filter_params: Optional[CaseUserFilter] = None, organization_ids: Optional[List[int]] = None
    ) -> List[User]:
        """Get users for a specific case with comprehensive filtering support.

        Args:
            case_id: The case ID to get users for
            filter_params: Optional CaseUserFilter with comprehensive filtering options
            organization_ids: Optional list of organization IDs (for backward compatibility)

        Returns:
            List of User objects

        Note: If both filter_params and organization_ids are provided, organization_ids in filter_params takes precedence.
        """
        # Handle backward compatibility
        if filter_params is None:
            filter_params = CaseUserFilter()

        # If organization_ids is provided and not set in filter_params, use it
        if organization_ids is not None and filter_params.organization_ids is None:
            filter_params.organization_ids = organization_ids

        query = GetCaseUsersQuery(self.http_client, case_id, filter_params)
        return query.execute()

    def check_name(self, name: str) -> bool:
        """Check if a case name is available."""
        query = CheckCaseNameQuery(self.http_client, name)
        return query.execute()

    # COMMANDS (Write operations)
    def create(self, case_data: CreateCaseRequest) -> Case:
        """Create a new case."""
        command = CreateCaseCommand(self.http_client, case_data)
        return command.execute()

    def update(self, case_id: str, update_data: UpdateCaseRequest) -> Case:
        """Update an existing case."""
        command = UpdateCaseCommand(self.http_client, case_id, update_data)
        return command.execute()

    def close(self, case_id: str) -> Case:
        """Close a case."""
        command = CloseCaseCommand(self.http_client, case_id)
        return command.execute()

    def open(self, case_id: str) -> Case:
        """Open a case."""
        command = OpenCaseCommand(self.http_client, case_id)
        return command.execute()

    def archive(self, case_id: str) -> Case:
        """Archive a case."""
        command = ArchiveCaseCommand(self.http_client, case_id)
        return command.execute()

    def change_owner(self, case_id: str, new_owner_id: str) -> Case:
        """Change case owner."""
        command = ChangeCaseOwnerCommand(self.http_client, case_id, new_owner_id)
        return command.execute()

    def remove_endpoints(self, case_id: str, filter_params: AssetFilter) -> Dict[str, Any]:
        """Remove endpoints from a case."""
        command = RemoveEndpointsFromCaseCommand(self.http_client, case_id, filter_params)
        return command.execute()

    def remove_task_assignment(self, case_id: str, task_assignment_id: str) -> Dict[str, Any]:
        """Remove task assignment from a case."""
        command = RemoveTaskAssignmentFromCaseCommand(self.http_client, case_id, task_assignment_id)
        return command.execute()

    def import_task_assignments(self, case_id: str, task_assignment_ids: List[str]) -> Dict[str, Any]:
        """Import task assignments to a case."""
        command = ImportTaskAssignmentsToCaseCommand(self.http_client, case_id, task_assignment_ids)
        return command.execute()

    def add_note(self, case_id: str, note_value: str) -> CaseNote:
        """Add a note to a case."""
        command = AddNoteToCaseCommand(self.http_client, case_id, note_value)
        return command.execute()

    def update_note(self, case_id: str, note_id: str, note_value: str) -> CaseNote:
        """Update a note in a case."""
        command = UpdateNoteToCaseCommand(self.http_client, case_id, note_id, note_value)
        return command.execute()

    def delete_note(self, case_id: str, note_id: str) -> Dict[str, Any]:
        """Delete a note from a case."""
        command = DeleteNoteToCaseCommand(self.http_client, case_id, note_id)
        return command.execute()

    def export_notes(self, case_id: str) -> Dict[str, Any]:
        """Export case notes as a file download (ZIP/CSV format)."""
        command = ExportCaseNotesCommand(self.http_client, case_id)
        return command.execute()

    def export_cases(self, filter_params: Optional[Dict[str, Any]] = None) -> Dict[str, Any]:
        """Export cases as a CSV file download."""
        command = ExportCasesCommand(self.http_client, filter_params)
        return command.execute()

    def export_endpoints(self, case_id: str, filter_params: Optional[Dict[str, Any]] = None) -> Dict[str, Any]:
        """Export case endpoints as a CSV file download with optional filtering."""
        command = ExportCaseEndpointsCommand(self.http_client, case_id, filter_params)
        return command.execute()

    def export_activities(self, case_id: str, filter_params: Optional[Dict[str, Any]] = None) -> Dict[str, Any]:
        """Export case activities as a CSV file download with optional filtering and pagination."""
        command = ExportCaseActivitiesCommand(self.http_client, case_id, filter_params)
        return command.execute()



"""
Auth API for the Binalyze AIR SDK.
"""

from typing import Dict, Any, Union

from ..http_client import HTTPClient
from ..models.auth import AuthStatus, LoginRequest, LoginResponse
from ..queries.auth import CheckAuthStatusQuery
from ..commands.auth import LoginCommand


class AuthAPI:
    """Auth API with CQRS pattern - separated queries and commands."""

    def __init__(self, http_client: HTTPClient):
        self.http_client = http_client

    # QUERIES (Read operations)
    def check_status(self) -> AuthStatus:
        """Check current authentication status."""
        query = CheckAuthStatusQuery(self.http_client)
        return query.execute()

    # COMMANDS (Write operations)
    def login(self, request: Union[LoginRequest, Dict[str, Any]]) -> LoginResponse:
        """Login user with credentials."""
        command = LoginCommand(self.http_client, request)
        return command.execute()



"""
Preset Filters queries for the Binalyze AIR SDK.
"""

from typing import Optional

from ..base import Query
from ..models.preset_filters import PresetFilter, PresetFiltersList, PresetFiltersFilter
from ..http_client import HTTPClient


class GetPresetFiltersQuery(Query[PresetFiltersList]):
    """Query to get preset filters."""
    
    def __init__(self, http_client: HTTPClient, filter_params: Optional[PresetFiltersFilter] = None):
        self.http_client = http_client
        self.filter_params = filter_params or PresetFiltersFilter()
    
    def execute(self) -> PresetFiltersList:
        """Execute the query to get preset filters."""
        params = {}
        
        # Add filter parameters
        if self.filter_params.organization_id is not None:
            params['filter[organizationId]'] = str(self.filter_params.organization_id)
        if self.filter_params.type:
            params['filter[type]'] = self.filter_params.type
        if self.filter_params.name:
            params['filter[name]'] = self.filter_params.name
        if self.filter_params.created_by:
            params['filter[createdBy]'] = self.filter_params.created_by
        
        # Add pagination parameters
        if self.filter_params.page_size:
            params['pageSize'] = str(self.filter_params.page_size)
        if self.filter_params.page_number:
            params['pageNumber'] = str(self.filter_params.page_number)
        if self.filter_params.sort_by:
            params['sortBy'] = self.filter_params.sort_by
        if self.filter_params.sort_type:
            params['sortType'] = self.filter_params.sort_type
        
        response = self.http_client.get('/preset-filters', params=params)
        return PresetFiltersList(**response['result'])


class GetPresetFilterByIdQuery(Query[Optional[PresetFilter]]):
    """Query to get a specific preset filter by ID."""
    
    def __init__(self, http_client: HTTPClient, filter_id: str):
        self.http_client = http_client
        self.filter_id = filter_id
    
    def execute(self) -> Optional[PresetFilter]:
        """Execute the query to get a preset filter by ID."""
        try:
            response = self.http_client.get(f'/preset-filters/{self.filter_id}')
            return PresetFilter(**response['result'])
        except Exception:
            return None 


"""
Triage-related commands for the Binalyze AIR SDK.
"""

from typing import Dict, Any, Union

from ..base import Command
from ..models.triage import (
    TriageRule, TriageTag, TriageProfile, CreateTriageRuleRequest,
    UpdateTriageRuleRequest, CreateTriageTagRequest, CreateTriageProfileRequest,
    TriageSeverity
)
from ..http_client import HTTPClient


class CreateTriageRuleCommand(Command[TriageRule]):
    """Command to create a new triage rule."""
    
    def __init__(self, http_client: HTTPClient, request: Union[CreateTriageRuleRequest, Dict[str, Any]]):
        self.http_client = http_client
        self.request = request
    
    def execute(self) -> TriageRule:
        """Execute the command to create a triage rule."""
        # Handle both dict and model objects
        if isinstance(self.request, dict):
            data = self.request.copy()
        else:
            # Convert SDK model fields to API fields
            request_dict = self.request.model_dump(exclude_none=True)
            data = {
                "description": request_dict.get("description") or request_dict.get("name", ""),
                "rule": request_dict.get("rule_content", ""),
                "engine": request_dict.get("type", "yara"),
                "searchIn": request_dict.get("search_in", "filesystem"),
                "organizationIds": [request_dict.get("organization_id", 0)]
            }
        
        response = self.http_client.post("triages/rules", json_data=data)
        
        entity_data = response.get("result", {})
        
        mapped_data = {
            "id": entity_data.get("_id"),
            "name": entity_data.get("description", ""),  # API uses description as name
            "description": entity_data.get("description"),
            "type": entity_data.get("engine"),  # API uses engine field
            "rule_content": entity_data.get("rule", ""),
            "search_in": entity_data.get("searchIn"),
            "enabled": entity_data.get("enabled", True),
            "severity": entity_data.get("severity", TriageSeverity.MEDIUM),
            "tags": entity_data.get("tags", []),
            "organization_id": entity_data.get("organizationIds", [0])[0] if entity_data.get("organizationIds") else 0,
            "organization_ids": entity_data.get("organizationIds", []),
            "created_at": entity_data.get("createdAt"),
            "updated_at": entity_data.get("updatedAt"),
            "created_by": entity_data.get("createdBy"),
            "updated_by": entity_data.get("updatedBy"),
            "match_count": entity_data.get("matchCount", 0),
            "last_match": entity_data.get("lastMatch"),
            "deletable": entity_data.get("deletable"),
        }
        
        # Remove None values
        mapped_data = {k: v for k, v in mapped_data.items() if v is not None}
        
        return TriageRule(**mapped_data)


class UpdateTriageRuleCommand(Command[TriageRule]):
    """Command to update an existing triage rule."""
    
    def __init__(self, http_client: HTTPClient, rule_id: str, request: Union[UpdateTriageRuleRequest, Dict[str, Any]]):
        self.http_client = http_client
        self.rule_id = rule_id
        self.request = request
    
    def execute(self) -> TriageRule:
        """Execute the command to update a triage rule."""
        # Handle both dict and model objects
        if isinstance(self.request, dict):
            data = self.request.copy()
        else:
            # Convert SDK model fields to API fields
            request_dict = self.request.model_dump(exclude_none=True)
            data = {}
            
            # Map SDK fields to API fields for update
            if "description" in request_dict or "name" in request_dict:
                data["description"] = request_dict.get("description") or request_dict.get("name", "")
            if "rule_content" in request_dict:
                data["rule"] = request_dict.get("rule_content")
            if "type" in request_dict:
                data["engine"] = request_dict.get("type")
            if "search_in" in request_dict:
                data["searchIn"] = request_dict.get("search_in")
            if "organization_id" in request_dict:
                data["organizationIds"] = [request_dict.get("organization_id")]
            if "enabled" in request_dict:
                data["enabled"] = request_dict.get("enabled")
        
        response = self.http_client.put(f"triages/rules/{self.rule_id}", json_data=data)
        
        entity_data = response.get("result", {})
        
        mapped_data = {
            "id": entity_data.get("_id"),
            "name": entity_data.get("description", ""),  # API uses description as name
            "description": entity_data.get("description"),
            "type": entity_data.get("engine"),  # API uses engine field
            "rule_content": entity_data.get("rule", ""),
            "search_in": entity_data.get("searchIn"),
            "enabled": entity_data.get("enabled", True),
            "severity": entity_data.get("severity", TriageSeverity.MEDIUM),
            "tags": entity_data.get("tags", []),
            "organization_id": entity_data.get("organizationIds", [0])[0] if entity_data.get("organizationIds") else 0,
            "organization_ids": entity_data.get("organizationIds", []),
            "created_at": entity_data.get("createdAt"),
            "updated_at": entity_data.get("updatedAt"),
            "created_by": entity_data.get("createdBy"),
            "updated_by": entity_data.get("updatedBy"),
            "match_count": entity_data.get("matchCount", 0),
            "last_match": entity_data.get("lastMatch"),
            "deletable": entity_data.get("deletable"),
        }
        
        # Remove None values
        mapped_data = {k: v for k, v in mapped_data.items() if v is not None}
        
        return TriageRule(**mapped_data)


class DeleteTriageRuleCommand(Command[Dict[str, Any]]):
    """Command to delete a triage rule."""
    
    def __init__(self, http_client: HTTPClient, rule_id: str):
        self.http_client = http_client
        self.rule_id = rule_id
    
    def execute(self) -> Dict[str, Any]:
        """Execute the command to delete a triage rule."""
        response = self.http_client.delete(f"triages/rules/{self.rule_id}")
        return response


class EnableTriageRuleCommand(Command[TriageRule]):
    """Command to enable a triage rule."""
    
    def __init__(self, http_client: HTTPClient, rule_id: str):
        self.http_client = http_client
        self.rule_id = rule_id
    
    def execute(self) -> TriageRule:
        """Execute the command to enable a triage rule."""
        data = {"enabled": True}
        
        response = self.http_client.put(f"triages/rules/{self.rule_id}", json_data=data)
        
        entity_data = response.get("result", {})
        
        mapped_data = {
            "id": entity_data.get("_id"),
            "name": entity_data.get("name"),
            "description": entity_data.get("description"),
            "type": entity_data.get("type"),
            "rule_content": entity_data.get("ruleContent", ""),
            "enabled": entity_data.get("enabled", True),
            "severity": entity_data.get("severity", TriageSeverity.MEDIUM),
            "tags": entity_data.get("tags", []),
            "organization_id": entity_data.get("organizationId", 0),
            "created_at": entity_data.get("createdAt"),
            "updated_at": entity_data.get("updatedAt"),
            "created_by": entity_data.get("createdBy"),
            "updated_by": entity_data.get("updatedBy"),
            "match_count": entity_data.get("matchCount", 0),
            "last_match": entity_data.get("lastMatch"),
        }
        
        # Remove None values
        mapped_data = {k: v for k, v in mapped_data.items() if v is not None}
        
        return TriageRule(**mapped_data)





class CreateTriageTagCommand(Command[TriageTag]):
    """Command to create a new triage tag."""
    
    def __init__(self, http_client: HTTPClient, request: Union[CreateTriageTagRequest, Dict[str, Any]]):
        self.http_client = http_client
        self.request = request
    
    def execute(self) -> TriageTag:
        """Execute the command to create a triage tag."""
        # Handle both dict and model objects
        if isinstance(self.request, dict):
            data = self.request
        else:
            data = self.request.model_dump(exclude_none=True, by_alias=True)
        
        response = self.http_client.post("triages/tags", json_data=data)
        
        entity_data = response.get("result", {})
        
        mapped_data = {
            "id": entity_data.get("id") or entity_data.get("_id"),  # Handle both id and _id
            "name": entity_data.get("name"),
            "description": entity_data.get("description"),
            "color": entity_data.get("color", "#3498db"),
            "organization_id": entity_data.get("organizationId", 0),
            "created_at": entity_data.get("createdAt"),
            "updated_at": entity_data.get("updatedAt"),
            "created_by": entity_data.get("createdBy", "Unknown"),  # Provide default for required field
            "usage_count": entity_data.get("usageCount") or entity_data.get("count", 0),  # Handle both count and usageCount
        }
        
        # Remove None values but keep defaults for required fields
        mapped_data = {k: v for k, v in mapped_data.items() if v is not None}
        
        return TriageTag(**mapped_data)


 


"""
Logging system for the Binalyze AIR SDK.

Provides comprehensive logging with different levels including HTTP trace logging.
"""

import logging
import sys
import json
import time
from typing import Optional, Dict, Any, List
from datetime import datetime
from pathlib import Path

from .constants import LogLevel


class SDKFormatter(logging.Formatter):
    """Custom formatter for SDK logging."""
    
    def __init__(self, include_timestamp: bool = True, include_level: bool = True):
        self.include_timestamp = include_timestamp
        self.include_level = include_level
        super().__init__()
    
    def format(self, record: logging.LogRecord) -> str:
        """Format log record with SDK-specific formatting."""
        parts = []
        
        if self.include_timestamp:
            timestamp = datetime.fromtimestamp(record.created).strftime('%Y-%m-%d %H:%M:%S.%f')[:-3]
            parts.append(f"[{timestamp}]")
        
        if self.include_level:
            parts.append(f"[{record.levelname}]")
        
        parts.append(f"[{record.name}]")
        parts.append(record.getMessage())
        
        return " ".join(parts)


class HTTPTraceLogger:
    """Logger specifically for HTTP request/response tracing."""
    
    def __init__(self, logger_name: str = "binalyze.air.http"):
        self.logger = logging.getLogger(logger_name)
        self.start_times: Dict[str, float] = {}
    
    def log_request(self, method: str, url: str, headers: Optional[Dict[str, str]] = None, 
                   body: Optional[Any] = None, request_id: Optional[str] = None) -> str:
        """Log HTTP request details."""
        if not request_id:
            request_id = f"req_{int(time.time() * 1000)}"
        
        self.start_times[request_id] = time.time()
        
        log_data: Dict[str, Any] = {
            "request_id": request_id,
            "method": method,
            "url": url,
            "timestamp": datetime.now().isoformat()
        }
        
        if headers:
            # Filter sensitive headers
            safe_headers = {k: v for k, v in headers.items() 
                          if k.lower() not in ['authorization', 'x-api-key', 'x-auth-token']}
            if safe_headers:
                log_data["headers"] = dict(safe_headers)
        
        if body and self.logger.isEnabledFor(logging.DEBUG):
            try:
                if isinstance(body, (dict, list)):
                    log_data["body"] = body
                elif isinstance(body, str):
                    # Try to parse as JSON for better formatting
                    try:
                        log_data["body"] = json.loads(body)
                    except json.JSONDecodeError:
                        log_data["body"] = body[:1000] + "..." if len(body) > 1000 else body
                else:
                    log_data["body"] = str(body)[:1000] + "..." if len(str(body)) > 1000 else str(body)
            except Exception:
                log_data["body"] = "<unable to serialize>"
        
        self.logger.info(f"HTTP Request: {json.dumps(log_data, indent=2)}")
        return request_id
    
    def log_response(self, request_id: str, status_code: int, headers: Optional[Dict[str, str]] = None,
                    body: Optional[Any] = None, error: Optional[Exception] = None):
        """Log HTTP response details."""
        duration = None
        if request_id in self.start_times:
            duration = time.time() - self.start_times[request_id]
            del self.start_times[request_id]
        
        log_data = {
            "request_id": request_id,
            "status_code": status_code,
            "timestamp": datetime.now().isoformat()
        }
        
        if duration is not None:
            log_data["duration_ms"] = round(duration * 1000, 2)
        
        if headers:
            safe_headers = {k: v for k, v in headers.items() 
                          if k.lower() not in ['set-cookie', 'authorization']}
            if safe_headers:
                log_data["headers"] = safe_headers
        
        if error:
            log_data["error"] = {
                "type": type(error).__name__,
                "message": str(error)
            }
            self.logger.error(f"HTTP Response (Error): {json.dumps(log_data, indent=2)}")
        else:
            if body and self.logger.isEnabledFor(logging.DEBUG):
                try:
                    if isinstance(body, (dict, list)):
                        log_data["body"] = body
                    elif isinstance(body, str):
                        try:
                            log_data["body"] = json.loads(body)
                        except json.JSONDecodeError:
                            log_data["body"] = body[:1000] + "..." if len(body) > 1000 else body
                    else:
                        log_data["body"] = str(body)[:1000] + "..." if len(str(body)) > 1000 else str(body)
                except Exception:
                    log_data["body"] = "<unable to serialize>"
            
            level = logging.INFO if status_code < 400 else logging.WARNING
            self.logger.log(level, f"HTTP Response: {json.dumps(log_data, indent=2)}")


class SDKLogger:
    """Main SDK logger with configuration options."""
    
    def __init__(self, name: str = "binalyze.air"):
        self.name = name
        self.logger = logging.getLogger(name)
        self.http_logger = HTTPTraceLogger()
        self._configured = False
    
    def configure(self, 
                 level: str = LogLevel.INFO,
                 enable_console: bool = True,
                 enable_file: bool = False,
                 file_path: Optional[str] = None,
                 enable_http_trace: bool = False,
                 format_json: bool = False,
                 max_file_size: int = 10 * 1024 * 1024,  # 10MB
                 backup_count: int = 5):
        """Configure the SDK logger."""
        # Allow reconfiguration for inheritance from global config
        # if self._configured:
        #     return
        
        # Clear existing handlers
        self.logger.handlers.clear()
        
        # Set level
        numeric_level = getattr(logging, level.upper(), logging.INFO)
        self.logger.setLevel(numeric_level)
        
        # Configure console handler
        if enable_console:
            console_handler = logging.StreamHandler(sys.stdout)
            console_handler.setLevel(numeric_level)
            
            if format_json:
                console_formatter = JSONFormatter()
            else:
                console_formatter = SDKFormatter()
            
            console_handler.setFormatter(console_formatter)
            self.logger.addHandler(console_handler)
        
        # Configure file handler
        if enable_file and file_path:
            from logging.handlers import RotatingFileHandler
            
            file_path_obj = Path(file_path)
            file_path_obj.parent.mkdir(parents=True, exist_ok=True)
            
            file_handler = RotatingFileHandler(
                file_path,
                maxBytes=max_file_size,
                backupCount=backup_count
            )
            file_handler.setLevel(numeric_level)
            
            if format_json:
                file_formatter = JSONFormatter()
            else:
                file_formatter = SDKFormatter()
            
            file_handler.setFormatter(file_formatter)
            self.logger.addHandler(file_handler)
        
        # Configure HTTP trace logging
        if enable_http_trace:
            self.http_logger.logger.setLevel(logging.DEBUG)
            # Add handlers to HTTP logger if not already present
            if not self.http_logger.logger.handlers:
                for handler in self.logger.handlers:
                    self.http_logger.logger.addHandler(handler)
        
        self._configured = True
    
    def debug(self, message: str, extra: Optional[Dict[str, Any]] = None):
        """Log debug message."""
        self.logger.debug(message, extra=extra or {})
    
    def info(self, message: str, extra: Optional[Dict[str, Any]] = None):
        """Log info message."""
        self.logger.info(message, extra=extra or {})
    
    def warning(self, message: str, extra: Optional[Dict[str, Any]] = None):
        """Log warning message."""
        self.logger.warning(message, extra=extra or {})
    
    def error(self, message: str, extra: Optional[Dict[str, Any]] = None):
        """Log error message."""
        self.logger.error(message, extra=extra or {})
    
    def critical(self, message: str, extra: Optional[Dict[str, Any]] = None):
        """Log critical message."""
        self.logger.critical(message, extra=extra or {})
    
    def log_api_call(self, api_name: str, method: str, endpoint: str, 
                    params: Optional[Dict[str, Any]] = None, 
                    success: bool = True, 
                    duration: Optional[float] = None,
                    response_size: Optional[int] = None):
        """Log API call information."""
        log_data = {
            "api_name": api_name,
            "method": method,
            "endpoint": endpoint,
            "success": success,
            "timestamp": datetime.now().isoformat()
        }
        
        if params:
            log_data["params"] = params
        
        if duration is not None:
            log_data["duration_ms"] = round(duration * 1000, 2)
        
        if response_size is not None:
            log_data["response_size_bytes"] = response_size
        
        level = logging.INFO if success else logging.ERROR
        self.logger.log(level, f"API Call: {json.dumps(log_data)}")
    
    def log_filter_usage(self, filter_type: str, filter_data: Dict[str, Any]):
        """Log filter usage for debugging."""
        log_data = {
            "filter_type": filter_type,
            "filter_data": filter_data,
            "timestamp": datetime.now().isoformat()
        }
        
        self.logger.debug(f"Filter Usage: {json.dumps(log_data)}")
    
    def get_http_tracer(self) -> HTTPTraceLogger:
        """Get HTTP trace logger."""
        return self.http_logger


class JSONFormatter(logging.Formatter):
    """JSON formatter for structured logging."""
    
    def format(self, record: logging.LogRecord) -> str:
        """Format log record as JSON."""
        log_data = {
            "timestamp": datetime.fromtimestamp(record.created).isoformat(),
            "level": record.levelname,
            "logger": record.name,
            "message": record.getMessage(),
        }
        
        # Add extra fields if present
        if hasattr(record, '__dict__'):
            extra_fields = {k: v for k, v in record.__dict__.items() 
                          if k not in ['name', 'msg', 'args', 'levelname', 'levelno', 'pathname', 
                                     'filename', 'module', 'lineno', 'funcName', 'created', 
                                     'msecs', 'relativeCreated', 'thread', 'threadName', 
                                     'processName', 'process', 'getMessage', 'exc_info', 'exc_text', 
                                     'stack_info', 'message']}
            if extra_fields:
                log_data.update(extra_fields)
        
        # Add exception info if present
        if record.exc_info:
            log_data["exception"] = self.formatException(record.exc_info)
        
        return json.dumps(log_data)


# Global logger instance and configuration
sdk_logger = SDKLogger()
_global_config = {}

# Convenience functions
def get_logger(name: Optional[str] = None) -> SDKLogger:
    """Get SDK logger instance that inherits global configuration."""
    if name:
        logger = SDKLogger(name)
        # Apply global configuration if it exists
        if _global_config:
            logger.configure(**_global_config)
        return logger
    return sdk_logger

def configure_logging(level: str = LogLevel.INFO,
                     enable_console: bool = True,
                     enable_file: bool = False,
                     file_path: Optional[str] = None,
                     enable_http_trace: bool = False,
                     format_json: bool = False,
                     verbose: bool = False,
                     debug: bool = False):
    """Configure SDK logging with convenience options."""
    global _global_config
    
    if debug:
        level = LogLevel.DEBUG
        enable_http_trace = True
    elif verbose:
        level = LogLevel.INFO
        enable_http_trace = True
    
    # Store global configuration for new loggers
    _global_config = {
        'level': level,
        'enable_console': enable_console,
        'enable_file': enable_file,
        'file_path': file_path,
        'enable_http_trace': enable_http_trace,
        'format_json': format_json
    }
    
    # Configure the main SDK logger
    sdk_logger.configure(**_global_config)

def enable_verbose_logging():
    """Enable verbose logging (INFO level with HTTP trace)."""
    configure_logging(
        level=LogLevel.INFO,
        enable_http_trace=True,
        verbose=True
    )

def enable_debug_logging():
    """Enable debug logging (DEBUG level with full HTTP trace)."""
    configure_logging(
        level=LogLevel.DEBUG,
        enable_http_trace=True,
        debug=True
    )

def disable_logging():
    """Disable all SDK logging."""
    sdk_logger.logger.setLevel(logging.CRITICAL + 1)
    sdk_logger.http_logger.logger.setLevel(logging.CRITICAL + 1)


# Performance logging utilities
class PerformanceTimer:
    """Context manager for timing operations."""
    
    def __init__(self, operation_name: str, logger: Optional[SDKLogger] = None):
        self.operation_name = operation_name
        self.logger = logger or sdk_logger
        self.start_time = None
        self.end_time = None
    
    def __enter__(self):
        self.start_time = time.time()
        self.logger.debug(f"Starting operation: {self.operation_name}")
        return self
    
    def __exit__(self, exc_type, exc_val, exc_tb):
        self.end_time = time.time()
        if self.start_time is not None:
            duration = self.end_time - self.start_time
            
            if exc_type is None:
                self.logger.info(f"Completed operation: {self.operation_name} in {duration:.3f}s")
            else:
                self.logger.error(f"Failed operation: {self.operation_name} after {duration:.3f}s - {exc_val}")
        else:
            self.logger.error(f"Failed operation: {self.operation_name} - start time not recorded")


def performance_timer(operation_name: str, logger: Optional[SDKLogger] = None):
    """Create a performance timer context manager."""
    return PerformanceTimer(operation_name, logger)


# Example usage:
"""
# Basic configuration
configure_logging(level=LogLevel.INFO, enable_console=True)

# Enable verbose logging
enable_verbose_logging()

# Enable debug logging with HTTP tracing
enable_debug_logging()

# Custom configuration
configure_logging(
    level=LogLevel.DEBUG,
    enable_console=True,
    enable_file=True,
    file_path="/tmp/binalyze_air_sdk.log",
    enable_http_trace=True,
    format_json=True
)

# Usage in code
logger = get_logger("my_module")
logger.info("Starting operation")

# Performance timing
with performance_timer("asset_retrieval"):
    assets = client.assets.get_assets()
""" 


"""
Audit Logs API for the Binalyze AIR SDK using CQRS pattern.
"""

from typing import List, Optional, Dict, Any
from ..http_client import HTTPClient
from ..models.audit import AuditLog, AuditLogsFilter
from ..queries.audit import ListAuditLogsQuery, ExportAuditLogsQuery


class AuditAPI:
    """Audit logs API aligned with official API specification."""
    
    def __init__(self, http_client: HTTPClient):
        self.http_client = http_client
    
    # QUERIES (Read operations)
    def list_logs(self, filter_params: Optional[AuditLogsFilter] = None, organization_ids: Optional[int] = None) -> List[AuditLog]:
        """List audit logs with filtering - Official API endpoint."""
        query = ListAuditLogsQuery(self.http_client, filter_params, organization_ids)
        return query.execute()
    
    def export_logs(self, filter_params: Optional[AuditLogsFilter] = None, format: str = "json", organization_ids: Optional[int] = None) -> Dict[str, Any]:
        """Export audit logs with filtering - Official API endpoint."""
        query = ExportAuditLogsQuery(self.http_client, filter_params, format, organization_ids)
        return query.execute() 


"""
Custom exceptions for the Binalyze AIR SDK.
"""

from typing import Optional, Dict, Any


class AIRAPIError(Exception):
    """Base exception for all AIR API errors."""
    
    def __init__(self, message: str, status_code: Optional[int] = None, response_data: Optional[Dict[str, Any]] = None):
        self.message = message
        self.status_code = status_code
        self.response_data = response_data or {}
        super().__init__(self.message)


class AuthenticationError(AIRAPIError):
    """Raised when authentication fails."""
    pass


class AuthorizationError(AIRAPIError):
    """Raised when authorization fails."""
    pass


class NotFoundError(AIRAPIError):
    """Raised when a resource is not found."""
    pass


class ValidationError(AIRAPIError):
    """Raised when request validation fails."""
    pass


class RateLimitError(AIRAPIError):
    """Raised when rate limit is exceeded."""
    pass


class ServerError(AIRAPIError):
    """Raised when server returns 5xx status codes."""
    pass


class NetworkError(AIRAPIError):
    """Raised when network-related errors occur."""
    pass 


"""
Params API models for the Binalyze AIR SDK.
"""

from typing import List, Optional, Dict, Any
from enum import Enum
from pydantic import Field

from ..base import AIRBaseModel


class ArtifactType(str, Enum):
    """Acquisition artifact types."""
    FILE = "file"
    REGISTRY = "registry"
    MEMORY = "memory"
    NETWORK = "network"
    PROCESS = "process"
    EVENT_LOG = "event_log"
    PREFETCH = "prefetch"
    BROWSER = "browser"
    SYSTEM = "system"


class ArtifactCategory(str, Enum):
    """Artifact categories."""
    FORENSICS = "forensics"
    MALWARE = "malware"
    NETWORK = "network"
    SYSTEM = "system"
    BROWSER = "browser"
    EMAIL = "email"
    CHAT = "chat"
    CLOUD = "cloud"


class Platform(str, Enum):
    """Supported platforms."""
    WINDOWS = "windows"
    LINUX = "linux"
    DARWIN = "darwin"
    MACOS = "macos"


class AcquisitionArtifact(AIRBaseModel):
    """Acquisition artifact model based on API response structure."""
    
    name: str
    desc: str = Field(alias="desc")
    type: str = Field(alias="type")
    
    # Additional fields for SDK processing
    group: Optional[str] = None
    platform: Optional[str] = None


class EDiscoveryPattern(AIRBaseModel):
    """E-Discovery pattern model based on API response structure."""
    
    name: str
    pattern: str
    
    # Additional fields for SDK processing  
    category: Optional[str] = None


class AcquisitionEvidence(AIRBaseModel):
    """Acquisition evidence model based on API response structure."""
    
    name: str
    desc: str = Field(alias="desc")
    type: str = Field(alias="type")
    
    # Additional fields for SDK processing
    group: Optional[str] = None
    platform: Optional[str] = None


class DroneAnalyzer(AIRBaseModel):
    """Drone analyzer model with proper field mapping."""
    
    id: str = Field(alias="Id")
    name: str = Field(alias="Name")
    default_enabled: bool = Field(alias="DefaultEnabled")
    platforms: List[str] = Field(default=[], alias="Platforms")
    o_ses: List[str] = Field(default=[], alias="OSes")
    
    # Computed properties can be added as methods if needed


# MITRE Attack models
class MitreAttackTactic(AIRBaseModel):
    """MITRE ATT&CK tactic model."""
    
    id: str
    name: str
    url: str


class MitreAttackTechnique(AIRBaseModel):
    """MITRE ATT&CK technique model."""
    
    id: str
    name: str
    url: str
    sub_techniques: Optional[List[str]] = Field(default=[], alias="subTechniques")
    parent_technique: Optional[str] = Field(default=None, alias="parentTechnique")


class MitreAttackResponse(AIRBaseModel):
    """MITRE ATT&CK response structure - matches actual API response format."""
    
    tactics: Dict[str, MitreAttackTactic]
    techniques: Dict[str, MitreAttackTechnique]


# API Response wrapper models for structured responses
class AcquisitionArtifactGroup(AIRBaseModel):
    """Group of acquisition artifacts."""
    
    group: str
    items: List[AcquisitionArtifact]


class AcquisitionArtifactsResponse(AIRBaseModel):
    """Full response structure for acquisition artifacts."""
    
    windows: List[AcquisitionArtifactGroup] = []
    linux: List[AcquisitionArtifactGroup] = []
    macos: List[AcquisitionArtifactGroup] = []
    aix: List[AcquisitionArtifactGroup] = []


class EDiscoveryCategory(AIRBaseModel):
    """E-Discovery pattern category."""
    
    category: str
    applications: List[EDiscoveryPattern]


class AcquisitionEvidenceGroup(AIRBaseModel):
    """Group of acquisition evidences."""
    
    group: str  
    items: List[AcquisitionEvidence]


class AcquisitionEvidencesResponse(AIRBaseModel):
    """Full response structure for acquisition evidences."""
    
    windows: List[AcquisitionEvidenceGroup] = []
    linux: List[AcquisitionEvidenceGroup] = []
    macos: List[AcquisitionEvidenceGroup] = []
    aix: List[AcquisitionEvidenceGroup] = [] 


from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name="binalyze",
    version="2.0.3",
    author="Binalyze",
    author_email="support@binalyze.com",
    description="Complete Python SDK for Binalyze Products - AIR, Fleet, and more",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/binalyze/python-sdk",
    packages=find_packages(exclude=["tests_*", "tests_*.*", "tools", "tools.*"]),
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Intended Audience :: Information Technology",
        "Intended Audience :: System Administrators",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Topic :: Security",
        "Topic :: System :: Systems Administration",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Software Development :: Libraries :: Application Frameworks",
    ],
    python_requires=">=3.8",
    install_requires=[
        "requests>=2.25.1",
        "pydantic>=2.0.0",
        "typing-extensions>=4.0.0",
        "python-dateutil>=2.8.0",
        "urllib3>=1.26.0",
    ],
    extras_require={
        "dev": [
            "pytest>=6.0",
            "pytest-asyncio",
            "black",
            "isort",
            "mypy",
            "flake8",
            "pylint",
        ],
        "testing": [
            "pytest>=6.0",
            "pytest-cov",
            "pytest-mock",
        ],
        "env": [
            "python-dotenv>=0.19.0",
        ],
        "full": [
            "python-dotenv>=0.19.0",
            "pytest>=6.0",
            "pytest-cov",
            "pytest-mock",
            "black",
            "isort",
            "mypy",
            "flake8",
            "pylint",
        ],
    },
    keywords="binalyze air fleet forensics security api sdk digital-forensics incident-response dfir",
    project_urls={
        "Bug Reports": "https://github.com/binalyze/python-sdk/issues",
        "Source": "https://github.com/binalyze/python-sdk",
        "Documentation": "https://github.com/binalyze/python-sdk/blob/main/README.md",
        "Binalyze": "https://binalyze.com",
        "AIR Product": "https://binalyze.com/products/air/",
    },
    entry_points={
        "console_scripts": [
            "binalyze-air=binalyze.air.cli:main",
        ],
    },
)



"""Multipart Upload query classes for Binalyze AIR SDK."""

from typing import Dict, Any, Optional
from ..models.multipart_upload import (
    UploadInitializeRequest, UploadPartRequest, UploadStatusRequest, UploadFinalizeRequest
)


class MultipartUploadQuery:
    """Base query class for multipart upload operations."""
    
    def __init__(self):
        """Initialize base multipart upload query."""
        self._params = {}

    def build_params(self) -> Dict[str, Any]:
        """Build query parameters.
        
        Returns:
            Dictionary of query parameters
        """
        return self._params.copy()


class InitializeUploadQuery(MultipartUploadQuery):
    """Query for initializing a multipart upload session.
    
    This query starts a new upload session and returns a unique file ID
    that will be used for all subsequent operations in this upload.
    """
    
    def __init__(self):
        """Initialize upload initialization query."""
        super().__init__()
        self._request = UploadInitializeRequest()

    def build_body(self) -> Dict[str, Any]:
        """Build request body for upload initialization.
        
        Returns:
            Dictionary containing request body (typically empty)
        """
        return self._request.to_dict()


class UploadPartQuery(MultipartUploadQuery):
    """Query for uploading a file part in a multipart upload.
    
    This query uploads a specific part of a file using the file ID
    obtained from the initialization step.
    """
    
    def __init__(self, file_id: str, part_number: int, file_data: bytes, filename: Optional[str] = None):
        """Initialize upload part query.
        
        Args:
            file_id: Upload session identifier
            part_number: Sequential part number (starting from 1)
            file_data: Binary file data for this part
            filename: Optional filename for the part
        """
        super().__init__()
        self._request = UploadPartRequest(
            file_id=file_id,
            part_number=part_number,
            file_data=file_data,
            filename=filename
        )

    def build_form_data(self) -> Dict[str, Any]:
        """Build form data for multipart file upload.
        
        Returns:
            Dictionary with form data fields
        """
        return self._request.to_form_data()

    @property
    def file_id(self) -> str:
        """Get the file ID for this upload.
        
        Returns:
            Upload session identifier
        """
        return self._request.file_id

    @property
    def part_number(self) -> int:
        """Get the part number for this upload.
        
        Returns:
            Part number being uploaded
        """
        return self._request.part_number

    @property
    def part_size(self) -> int:
        """Get the size of this part.
        
        Returns:
            Size of the part in bytes
        """
        return self._request.get_part_size()

    @property
    def part_hash(self) -> str:
        """Get MD5 hash of this part.
        
        Returns:
            MD5 hash of the part data
        """
        return self._request.get_part_hash()


class CheckUploadStatusQuery(MultipartUploadQuery):
    """Query for checking the status of a multipart upload.
    
    This query checks if all parts have been uploaded and the upload
    is ready to be finalized.
    """
    
    def __init__(self, file_id: str):
        """Initialize upload status check query.
        
        Args:
            file_id: Upload session identifier to check
        """
        super().__init__()
        self._request = UploadStatusRequest(file_id=file_id)

    def build_params(self) -> Dict[str, Any]:
        """Build query parameters for status check.
        
        Returns:
            Dictionary containing query parameters
        """
        return self._request.to_dict()

    @property
    def file_id(self) -> str:
        """Get the file ID being checked.
        
        Returns:
            Upload session identifier
        """
        return self._request.file_id


class FinalizeUploadQuery(MultipartUploadQuery):
    """Query for finalizing a multipart upload.
    
    This query completes the upload process by combining all uploaded
    parts into the final file.
    """
    
    def __init__(self, file_id: str):
        """Initialize upload finalization query.
        
        Args:
            file_id: Upload session identifier to finalize
        """
        super().__init__()
        self._request = UploadFinalizeRequest(file_id=file_id)

    def build_body(self) -> Dict[str, Any]:
        """Build request body for upload finalization.
        
        Returns:
            Dictionary containing request body
        """
        return self._request.to_dict()

    @property
    def file_id(self) -> str:
        """Get the file ID being finalized.
        
        Returns:
            Upload session identifier
        """
        return self._request.file_id 


"""
Acquisition-related queries for the Binalyze AIR SDK.
"""

from typing import List, Optional

from ..base import Query
from ..constants import AssetPlatform
from ..models.acquisitions import (
    AcquisitionProfile, AcquisitionProfileDetails, AcquisitionFilter,
    AcquisitionProfilePlatformDetails, NetworkCaptureConfig, EDiscoveryPattern
)
from ..http_client import HTTPClient


class ListAcquisitionProfilesQuery(Query[List[AcquisitionProfile]]):
    """Query to list acquisition profiles with optional filtering."""
    
    def __init__(
        self, 
        http_client: HTTPClient, 
        filter_params: Optional[AcquisitionFilter] = None,
        organization_ids: Optional[List[int]] = None,
        all_organizations: bool = False
    ):
        self.http_client = http_client
        # Initialize filter with default organization IDs if not provided
        if filter_params is None:
            filter_params = AcquisitionFilter()
        
        # Set organization parameters if not already set in filter
        if filter_params.organization_ids is None and organization_ids is not None:
            filter_params.organization_ids = organization_ids
        elif filter_params.organization_ids is None:
            filter_params.organization_ids = [0]  # Default to organization 0
        
        # Set all_organizations parameter if not already set in filter
        if filter_params.all_organizations is None and all_organizations:
            filter_params.all_organizations = all_organizations
        
        self.filter_params = filter_params
    
    def execute(self) -> List[AcquisitionProfile]:
        """Execute the query to list acquisition profiles."""
        # Use filter's parameter generation
        params = self.filter_params.to_params()
        
        response = self.http_client.get("acquisitions/profiles", params=params)
        
        entities = response.get("result", {}).get("entities", [])
        
        # Convert to AcquisitionProfile objects
        profiles = []
        for entity_data in entities:
            mapped_data = {
                "id": entity_data.get("_id"),
                "name": entity_data.get("name"),
                "organization_ids": entity_data.get("organizationIds", []),  # Keep as integers
                "created_at": entity_data.get("createdAt"),
                "updated_at": entity_data.get("updatedAt"),
                "created_by": entity_data.get("createdBy"),
                "deletable": entity_data.get("deletable", True),
                "average_time": entity_data.get("averageTime"),
                "last_used_at": entity_data.get("lastUsedAt"),
                "last_used_by": entity_data.get("lastUsedBy"),
                "has_event_log_records_evidence": entity_data.get("hasEventLogRecordsEvidence"),
            }
            
            # Remove None values
            mapped_data = {k: v for k, v in mapped_data.items() if v is not None}
            
            profiles.append(AcquisitionProfile(**mapped_data))
        
        return profiles


class GetAcquisitionProfileQuery(Query[AcquisitionProfileDetails]):
    """Query to get a specific acquisition profile by ID."""
    
    def __init__(self, http_client: HTTPClient, profile_id: str):
        self.http_client = http_client
        self.profile_id = profile_id
    
    def execute(self) -> AcquisitionProfileDetails:
        """Execute the query to get acquisition profile details."""
        response = self.http_client.get(f"acquisitions/profiles/{self.profile_id}")
        
        entity_data = response.get("result", {})
        
        # Parse platform configurations
        def parse_platform_config(platform_data: dict) -> AcquisitionProfilePlatformDetails:
            network_capture = None
            if "networkCapture" in platform_data:
                nc_data = platform_data["networkCapture"]
                network_capture = NetworkCaptureConfig(
                    enabled=nc_data.get("enabled", False),
                    duration=nc_data.get("duration", 60),
                    pcap=nc_data.get("pcap", {"enabled": False}),
                    networkFlow=nc_data.get("networkFlow", {"enabled": False})
                )
            
            return AcquisitionProfilePlatformDetails(
                evidenceList=platform_data.get("evidenceList", []),
                artifactList=platform_data.get("artifactList"),
                customContentProfiles=platform_data.get("customContentProfiles", []),
                networkCapture=network_capture
            )
        
        # Parse platform configurations
        windows_config = None
        linux_config = None
        macos_config = None
        aix_config = None
        
        if AssetPlatform.WINDOWS in entity_data:
            windows_config = parse_platform_config(entity_data[AssetPlatform.WINDOWS])
        
        if AssetPlatform.LINUX in entity_data:
            linux_config = parse_platform_config(entity_data[AssetPlatform.LINUX])
        
        if AssetPlatform.DARWIN in entity_data:
            macos_config = parse_platform_config(entity_data[AssetPlatform.DARWIN])
        
        if AssetPlatform.AIX in entity_data:
            aix_config = parse_platform_config(entity_data[AssetPlatform.AIX])
        
        # Parse eDiscovery patterns
        e_discovery = None
        if "eDiscovery" in entity_data and "patterns" in entity_data["eDiscovery"]:
            patterns = [
                EDiscoveryPattern(
                    pattern=pattern.get("pattern", ""),
                    category=pattern.get("category", "")
                )
                for pattern in entity_data["eDiscovery"]["patterns"]
            ]
            e_discovery = {"patterns": patterns}
        
        mapped_data = {
            "id": entity_data.get("_id"),
            "name": entity_data.get("name"),
            "organization_ids": [str(org_id) for org_id in entity_data.get("organizationIds", [])],
            "created_at": entity_data.get("createdAt"),
            "updated_at": entity_data.get("updatedAt"),
            "created_by": entity_data.get("createdBy"),
            "deletable": entity_data.get("deletable", True),
            AssetPlatform.WINDOWS: windows_config,
            AssetPlatform.LINUX: linux_config,
            AssetPlatform.DARWIN: macos_config,
            AssetPlatform.AIX: aix_config,
            "e_discovery": e_discovery,
        }
        
        # Remove None values
        mapped_data = {k: v for k, v in mapped_data.items() if v is not None}
        
        return AcquisitionProfileDetails(**mapped_data) 


"""
Baseline-related queries for the Binalyze AIR SDK.
"""

from typing import Dict, Any

from ..base import Query
from ..http_client import HTTPClient


class GetBaselineComparisonReportQuery(Query[Dict[str, Any]]):
    """Query to get baseline comparison report by endpoint ID and task ID."""
    
    def __init__(self, http_client: HTTPClient, endpoint_id: str, task_id: str):
        self.http_client = http_client
        self.endpoint_id = endpoint_id
        self.task_id = task_id
    
    def execute(self) -> Dict[str, Any]:
        """Execute the query to get baseline comparison report."""
        return self.http_client.get(f"baseline/comparison/report/{self.endpoint_id}/{self.task_id}") 


"""
Notifications API for the Binalyze AIR SDK.
"""

from typing import Optional, Union, Dict, Any

from ..http_client import HTTPClient
from ..models.notifications import Notification, NotificationsList, NotificationsFilter, NotificationType, NotificationLevel
from ..queries.notifications import GetNotificationsQuery, GetNotificationByIdQuery
from ..commands.notifications import DeleteAllNotificationsCommand, MarkAllAsReadCommand, MarkAsReadByIdCommand


class NotificationsAPI:
    """Notifications API with CQRS pattern - separated queries and commands."""
    
    def __init__(self, http_client: HTTPClient):
        self.http_client = http_client
    
    # QUERIES (Read operations)
    def get_notifications(self, filter_params: Optional[NotificationsFilter] = None) -> NotificationsList:
        """Get notifications with optional filtering."""
        query = GetNotificationsQuery(self.http_client, filter_params)
        return query.execute()
    
    def get_notification_by_id(self, notification_id: Union[int, str]) -> Optional[Notification]:
        """Get a specific notification by ID."""
        query = GetNotificationByIdQuery(self.http_client, str(notification_id))
        return query.execute()
    
    # Convenience methods for common queries
    def get_unread_notifications(self, organization_ids: Union[int, list] = 0) -> NotificationsList:
        """Get unread notifications."""
        # Create filter with explicit field values
        filter_params = NotificationsFilter()
        filter_params.is_read = False
        if isinstance(organization_ids, list):
            filter_params.organization_ids = organization_ids
        else:
            filter_params.organization_ids = organization_ids
        
        return self.get_notifications(filter_params)
    
    def get_notifications_by_type(self, notification_type: Union[str, NotificationType], organization_ids: Union[int, list] = 0) -> NotificationsList:
        """Get notifications by type."""
        # Create filter with explicit field values
        filter_params = NotificationsFilter()
        if isinstance(notification_type, str):
            filter_params.type = NotificationType(notification_type)
        else:
            filter_params.type = notification_type
        if isinstance(organization_ids, list):
            filter_params.organization_ids = organization_ids
        else:
            filter_params.organization_ids = organization_ids
        
        return self.get_notifications(filter_params)
    
    def get_notifications_by_level(self, level: Union[str, NotificationLevel], organization_ids: Union[int, list] = 0) -> NotificationsList:
        """Get notifications by level."""
        # Create filter with explicit field values  
        filter_params = NotificationsFilter()
        if isinstance(level, str):
            filter_params.level = NotificationLevel(level)
        else:
            filter_params.level = level
        if isinstance(organization_ids, list):
            filter_params.organization_ids = organization_ids
        else:
            filter_params.organization_ids = organization_ids
        
        return self.get_notifications(filter_params)
    
    # COMMANDS (Write operations)
    def delete_all_notifications(self) -> Dict[str, Any]:
        """Delete all notifications for the current user."""
        command = DeleteAllNotificationsCommand(self.http_client)
        return command.execute()
    
    def mark_all_as_read(self) -> Dict[str, Any]:
        """Mark all notifications as read."""
        command = MarkAllAsReadCommand(self.http_client)
        return command.execute()
    
    def mark_as_read_by_id(self, notification_id: Union[int, str]) -> Dict[str, Any]:
        """Mark a specific notification as read by ID."""
        command = MarkAsReadByIdCommand(self.http_client, notification_id)
        return command.execute()
    
    # Convenience methods
    def get_notification_count(self, organization_ids: Union[int, list] = 0) -> int:
        """Get total notification count."""
        # Create filter with explicit field values
        filter_params = NotificationsFilter()
        if isinstance(organization_ids, list):
            filter_params.organization_ids = organization_ids
        else:
            filter_params.organization_ids = organization_ids
        filter_params.page_size = 1  # We only need the count
        
        result = self.get_notifications(filter_params)
        return result.total_entity_count
    
    def get_unread_count(self, organization_ids: Union[int, list] = 0) -> int:
        """Get unread notification count."""
        # Create filter with explicit field values
        filter_params = NotificationsFilter()
        filter_params.is_read = False
        if isinstance(organization_ids, list):
            filter_params.organization_ids = organization_ids
        else:
            filter_params.organization_ids = organization_ids
        filter_params.page_size = 1  # We only need the count
        
        result = self.get_notifications(filter_params)
        return result.total_entity_count 


"""
Relay Server models for the Binalyze AIR SDK.
"""

from typing import Optional, List, Dict, Any
from datetime import datetime

from ..base import AIRBaseModel, Filter


class RelayServer(AIRBaseModel):
    """Relay server model."""
    
    id: int
    name: str
    endpoint_count: int
    last_seen: datetime
    online_status: str
    label: str
    address: str


class RelayServersList(AIRBaseModel):
    """Relay servers list response model."""
    
    entities: List[RelayServer]
    total_entity_count: Optional[int] = None
    current_page: Optional[int] = None
    page_size: Optional[int] = None
    previous_page: Optional[int] = None
    total_page_count: Optional[int] = None
    next_page: Optional[int] = None
    filters: Optional[List[Dict[str, Any]]] = None
    sortables: Optional[List[str]] = None


class RelayServersFilter(Filter):
    """Filter parameters for relay servers queries."""
    
    # Override the default organization_ids to use singular organizationId
    organization_ids: Optional[List[int]] = None
    
    # Add organizationId (singular) as required by the API
    organization_id: Optional[int] = None
    
    name: Optional[str] = None
    online_status: Optional[str] = None
    
    def to_params(self) -> Dict[str, Any]:
        """Convert filter to API parameters, using organizationId (singular) for relay servers."""
        params = {}
        
        # Pagination parameters (not in filter namespace)
        if self.page_number is not None:
            params["pageNumber"] = self.page_number
        if self.page_size is not None:
            params["pageSize"] = self.page_size
        if self.sort_by is not None:
            params["sortBy"] = self.sort_by
        if self.sort_type is not None:
            params["sortType"] = self.sort_type
        
        # Always add organizationId (required by API) - default to 0 if not set
        organization_id = self.organization_id if self.organization_id is not None else 0
        params["filter[organizationId]"] = str(organization_id)
        
        # Filter parameters (in filter namespace) - EXCLUDE organization_ids and organization_id
        for field_name, field_value in self.model_dump(exclude_none=True).items():
            # Skip pagination/sorting fields and organization fields (handled above)
            if field_name in ["page_number", "page_size", "sort_by", "sort_type", "organization_ids", "organization_id"]:
                continue
                
            if field_value is not None:
                if isinstance(field_value, list):
                    if len(field_value) > 0:  # Only add non-empty lists
                        params[f"filter[{field_name}]"] = ",".join([str(x) for x in field_value])
                else:
                    params[f"filter[{field_name}]"] = str(field_value)
        return params


class RebootTaskRequest(AIRBaseModel):
    """Request model for reboot task."""
    
    endpoint_filter: Dict[str, Any]


class ShutdownTaskRequest(AIRBaseModel):
    """Request model for shutdown task."""
    
    endpoint_filter: Dict[str, Any]


class LogRetrievalTaskRequest(AIRBaseModel):
    """Request model for log retrieval task."""
    
    endpoint_filter: Dict[str, Any]


class VersionUpdateTaskRequest(AIRBaseModel):
    """Request model for version update task."""
    
    endpoint_filter: Dict[str, Any]


class UpdateTagsRequest(AIRBaseModel):
    """Request model for updating tags."""
    
    tags: List[str]


class UpdateLabelRequest(AIRBaseModel):
    """Request model for updating label."""
    
    label: str


class UpdateAddressRequest(AIRBaseModel):
    """Request model for updating address."""
    
    address: str 


"""Logger query classes for Binalyze AIR SDK."""

from typing import Dict, Any, Optional
from ..models.logger import LogDownloadRequest


class LoggerQuery:
    """Base query class for logger operations."""
    
    def __init__(self):
        """Initialize base logger query."""
        self._params = {}

    def build_params(self) -> Dict[str, Any]:
        """Build query parameters.
        
        Returns:
            Dictionary of query parameters
        """
        return self._params.copy()


class DownloadLogsQuery(LoggerQuery):
    """Query for downloading application logs as ZIP file.
    
    This query downloads system logs which can include:
    - Application logs
    - Error logs
    - System diagnostic information
    - Debug logs (if enabled)
    """
    
    def __init__(self, latest_log_file: bool = False):
        """Initialize download logs query.
        
        Args:
            latest_log_file: Whether to download only the latest log file
        """
        super().__init__()
        self._latest_log_file = latest_log_file

    def build_params(self) -> Dict[str, Any]:
        """Build parameters for download logs request.
        
        Returns:
            Dictionary containing query parameters
        """
        request = LogDownloadRequest(latest_log_file=self._latest_log_file)
        return request.to_dict()

    @property
    def latest_log_file(self) -> bool:
        """Get the latest log file flag.
        
        Returns:
            Whether to download only the latest log file
        """
        return self._latest_log_file 


"""
Evidences/Repositories-related commands for the Binalyze AIR SDK.
"""

from typing import Dict, Any

from ..base import Command
from ..models.evidences import (
    EvidenceRepository, AmazonS3Repository, AzureStorageRepository,
    FTPSRepository, SFTPRepository, SMBRepository,
    CreateAmazonS3RepositoryRequest, UpdateAmazonS3RepositoryRequest,
    CreateAzureStorageRepositoryRequest, UpdateAzureStorageRepositoryRequest,
    CreateFTPSRepositoryRequest, UpdateFTPSRepositoryRequest,
    CreateSFTPRepositoryRequest, UpdateSFTPRepositoryRequest,
    CreateSMBRepositoryRequest, UpdateSMBRepositoryRequest,
    ValidateRepositoryRequest, ValidationResult
)
from ..http_client import HTTPClient


# General Repository Commands

class UpdateRepositoryCommand(Command[EvidenceRepository]):
    """Command to update evidence repository."""

    def __init__(self, http_client: HTTPClient, repository_id: str, update_data: Dict[str, Any]):
        self.http_client = http_client
        self.repository_id = repository_id
        self.update_data = update_data

    def execute(self) -> EvidenceRepository:
        """Execute the update repository command."""
        response = self.http_client.put(
            f"evidences/repositories/{self.repository_id}",
            json_data=self.update_data
        )

        if response.get("success"):
            repository_data = response.get("result", {})
            return EvidenceRepository(**repository_data)

        raise Exception(f"Failed to update repository: {response.get('error', 'Unknown error')}")


class DeleteRepositoryCommand(Command[Dict[str, Any]]):
    """Command to delete evidence repository."""

    def __init__(self, http_client: HTTPClient, repository_id: str):
        self.http_client = http_client
        self.repository_id = repository_id

    def execute(self) -> Dict[str, Any]:
        """Execute the delete repository command."""
        try:
            response = self.http_client.delete(f"evidences/repositories/{self.repository_id}")

            if response.get("success"):
                return response

            # Handle error response with detailed information from API
            errors = response.get("errors", [])
            status_code = response.get("statusCode", "Unknown")
            error_message = "; ".join(errors) if errors else response.get("error", "Unknown error")

            raise Exception(f"Failed to delete repository (HTTP {status_code}): {error_message}")

        except Exception as e:
            # Check if this is already our formatted exception
            if "Failed to delete repository" in str(e):
                raise e

            # Handle HTTP client exceptions and format them consistently
            error_str = str(e)
            if "404" in error_str or "not found" in error_str.lower():
                raise Exception("Failed to delete repository (HTTP 404): No evidence repository found by provided id(s)")
            else:
                raise Exception(f"Failed to delete repository: {error_str}")


# Amazon S3 Repository Commands

class CreateAmazonS3RepositoryCommand(Command[AmazonS3Repository]):
    """Command to create Amazon S3 repository."""

    def __init__(self, http_client: HTTPClient, request: CreateAmazonS3RepositoryRequest):
        self.http_client = http_client
        self.request = request

    def execute(self) -> AmazonS3Repository:
        """Execute the create Amazon S3 repository command."""
        payload = self.request.model_dump(exclude_none=True)

        # Handle field mapping for API compatibility
        if 'organizationId' in payload:
            # API expects organizationIds array, not organizationId
            if 'organizationIds' not in payload or not payload['organizationIds']:
                payload['organizationIds'] = [payload['organizationId']]
            del payload['organizationId']

        # Map SDK field bucketName to API field bucket
        if 'bucketName' in payload:
            payload['bucket'] = payload['bucketName']
            del payload['bucketName']

        # Remove SDK-specific fields that API doesn't expect
        if 'isDefault' in payload:
            del payload['isDefault']

        response = self.http_client.post("evidences/repositories/amazon-s3", json_data=payload)

        if response.get("success"):
            repository_data = response.get("result", {})

            # Handle field mapping from API response back to SDK model
            if '_id' in repository_data:
                repository_data['id'] = repository_data['_id']
                del repository_data['_id']

            # Map organizationIds array back to organizationId for SDK model
            if 'organizationIds' in repository_data and repository_data['organizationIds']:
                repository_data['organizationId'] = repository_data['organizationIds'][0]

            # Map API field bucket back to SDK field bucketName
            if 'bucket' in repository_data:
                repository_data['bucketName'] = repository_data['bucket']
                del repository_data['bucket']

            return AmazonS3Repository(**repository_data)

        raise Exception(f"Failed to create Amazon S3 repository: {response.get('error', 'Unknown error')}")


class UpdateAmazonS3RepositoryCommand(Command[AmazonS3Repository]):
    """Command to update Amazon S3 repository."""

    def __init__(self, http_client: HTTPClient, repository_id: str, request: UpdateAmazonS3RepositoryRequest):
        self.http_client = http_client
        self.repository_id = repository_id
        self.request = request

    def execute(self) -> AmazonS3Repository:
        """Execute the update Amazon S3 repository command."""
        # The Amazon S3 update API requires a complete payload, not partial updates
        # First, get the current repository data
        current_response = self.http_client.get(f"evidences/repositories/{self.repository_id}")

        if not current_response.get("success"):
            raise Exception(f"Failed to get current repository data: {current_response.get('error', 'Unknown error')}")

        current_data = current_response.get("result", {})

        # Create update payload by merging current data with updates
        update_data = self.request.model_dump()  # Don't exclude None to see all fields

        # Start with required fields from current repository
        payload = {
            "name": current_data.get("name", ""),
            "region": current_data.get("region", ""),
            "bucket": current_data.get("bucket", ""),  # API uses 'bucket'
            "accessKeyId": current_data.get("accessKeyId", ""),
            "secretAccessKey": current_data.get("secretAccessKey", ""),
            "organizationIds": current_data.get("organizationIds", [0])
        }

        # Apply updates from the request
        for key, value in update_data.items():
            if key == "organizationId" and value is not None:
                # Convert organizationId to organizationIds array for API
                payload["organizationIds"] = [value]
            elif key == "bucketName" and value is not None:
                # Map SDK field bucketName to API field bucket
                payload["bucket"] = value
            elif key in ["name", "region", "accessKeyId", "secretAccessKey"] and value is not None:
                payload[key] = value
            # Skip other SDK-specific fields that don't map to API

        response = self.http_client.put(
            f"evidences/repositories/amazon-s3/{self.repository_id}",
            json_data=payload
        )

        if response.get("success"):
            repository_data = response.get("result", {})

            # Handle field mapping from API response back to SDK model
            if '_id' in repository_data:
                repository_data['id'] = repository_data['_id']
                del repository_data['_id']

            # Map organizationIds array back to organizationId for SDK model
            if 'organizationIds' in repository_data and repository_data['organizationIds']:
                repository_data['organizationId'] = repository_data['organizationIds'][0]

            # Map API field bucket back to SDK field bucketName
            if 'bucket' in repository_data:
                repository_data['bucketName'] = repository_data['bucket']
                del repository_data['bucket']

            return AmazonS3Repository(**repository_data)

        raise Exception(f"Failed to update Amazon S3 repository: {response.get('error', 'Unknown error')}")





class ValidateAmazonS3RepositoryCommand(Command[ValidationResult]):
    """Command to validate Amazon S3 repository."""

    def __init__(self, http_client: HTTPClient, request: ValidateRepositoryRequest):
        self.http_client = http_client
        self.request = request

    def execute(self) -> ValidationResult:
        """Execute the validate Amazon S3 repository command."""
        payload = self.request.model_dump(exclude_none=True)

        # The validation API expects the same field structure as create API
        # Extract config if using the generic ValidateRepositoryRequest structure
        if 'config' in payload and isinstance(payload['config'], dict):
            payload = payload['config']

        # Apply the same field mapping as CreateAmazonS3RepositoryCommand
        # Handle field mapping for API compatibility
        if 'organizationId' in payload:
            # API expects organizationIds array, not organizationId
            if 'organizationIds' not in payload or not payload['organizationIds']:
                payload['organizationIds'] = [payload['organizationId']]
            del payload['organizationId']

        # Map SDK field bucketName to API field bucket
        if 'bucketName' in payload:
            payload['bucket'] = payload['bucketName']
            del payload['bucketName']

        # Remove SDK-specific fields that API doesn't expect
        if 'isDefault' in payload:
            del payload['isDefault']

        try:
            response = self.http_client.post("evidences/repositories/validate/amazon-s3", json_data=payload)
        except Exception as e:
            # Handle specific validation errors (like 603) as successful validation responses
            error_str = str(e)
            if "603" in error_str:
                # 603 means validation is working, but credentials are invalid
                # This is a successful validation result
                return ValidationResult(
                    isValid=False,
                    message="AWS Access Key validation failed - credentials invalid but validation working"
                )
            else:
                # Re-raise other exceptions
                raise e

        if response.get("success"):
            validation_data = response.get("result", {})
            return ValidationResult(**validation_data)

        # Handle validation errors as successful validation responses
        status_code = response.get("statusCode")
        if status_code == 603:
            # AWS validation failed but validation is working
            return ValidationResult(
                isValid=False,
                message="AWS Access Key validation failed - credentials invalid but validation working"
            )

        raise Exception(f"Failed to validate Amazon S3 repository: {response.get('error', 'Unknown error')}")


# Azure Storage Repository Commands

class CreateAzureStorageRepositoryCommand(Command[AzureStorageRepository]):
    """Command to create Azure Storage repository."""

    def __init__(self, http_client: HTTPClient, request: CreateAzureStorageRepositoryRequest):
        self.http_client = http_client
        self.request = request

    def execute(self) -> AzureStorageRepository:
        """Execute the create Azure Storage repository command."""
        payload = self.request.model_dump(exclude_none=True)

        # Handle field mapping for API compatibility
        if 'organizationId' in payload:
            # API expects organizationIds array, not organizationId
            if 'organizationIds' not in payload or not payload['organizationIds']:
                payload['organizationIds'] = [payload['organizationId']]
            del payload['organizationId']

        # Convert SDK Azure Storage fields to API SASUrl format
        if 'accountName' in payload and 'accountKey' in payload and 'containerName' in payload:
            # Build a basic SAS URL format from the individual components
            account_name = payload['accountName']
            container_name = payload['containerName']
            # Create a test SAS URL format (this would normally come from Azure)
            sas_url = f"https://{account_name}.blob.core.windows.net/{container_name}?sv=2022-01-01&ss=b&srt=co&sp=rwdlacupx&se=2025-12-31T23:59:59Z&st=2024-01-01T00:00:00Z&spr=https&sig=test"
            payload['SASUrl'] = sas_url

            # Remove individual fields that API doesn't expect
            del payload['accountName']
            del payload['accountKey']
            del payload['containerName']

        # Remove SDK-specific fields that API doesn't expect
        if 'isDefault' in payload:
            del payload['isDefault']

        response = self.http_client.post("evidences/repositories/azure-storage", json_data=payload)

        if response.get("success"):
            repository_data = response.get("result", {})

            # Handle field mapping from API response back to SDK model
            if '_id' in repository_data:
                repository_data['id'] = repository_data['_id']
                del repository_data['_id']

            # Map organizationIds array back to organizationId for SDK model
            if 'organizationIds' in repository_data and repository_data['organizationIds']:
                repository_data['organizationId'] = repository_data['organizationIds'][0]

            # Extract Azure Storage fields from SASUrl for SDK model
            if 'SASUrl' in repository_data:
                sas_url = repository_data['SASUrl']
                # Parse accountName and containerName from SAS URL
                if sas_url and '://' in sas_url:
                    try:
                        # Extract account name from URL like https://accountname.blob.core.windows.net/container...
                        url_parts = sas_url.split('://', 1)[1].split('/')
                        if len(url_parts) >= 2:
                            domain_parts = url_parts[0].split('.')
                            if len(domain_parts) >= 1:
                                repository_data['accountName'] = domain_parts[0]
                            repository_data['containerName'] = url_parts[1].split('?')[0]
                    except (ValueError, IndexError, AttributeError):
                        # Fallback values if URL parsing fails
                        repository_data['accountName'] = 'azure-storage'
                        repository_data['containerName'] = 'container'

                # Provide a placeholder for accountKey since API doesn't return it
                repository_data['accountKey'] = 'hidden'

            return AzureStorageRepository(**repository_data)

        raise Exception(f"Failed to create Azure Storage repository: {response.get('error', 'Unknown error')}")


class UpdateAzureStorageRepositoryCommand(Command[AzureStorageRepository]):
    """Command to update Azure Storage repository."""

    def __init__(self, http_client: HTTPClient, repository_id: str, request: UpdateAzureStorageRepositoryRequest):
        self.http_client = http_client
        self.repository_id = repository_id
        self.request = request

    def execute(self) -> AzureStorageRepository:
        """Execute the update Azure Storage repository command."""
        # The Azure Storage update API requires a complete payload, not partial updates
        # First, get the current repository data
        current_response = self.http_client.get(f"evidences/repositories/{self.repository_id}")

        if not current_response.get("success"):
            raise Exception(f"Failed to get current repository data: {current_response.get('error', 'Unknown error')}")

        current_data = current_response.get("result", {})

        # Create update payload by merging current data with updates
        update_data = self.request.model_dump()  # Don't exclude None to see all fields

        # Start with current data and build API payload format
        payload = {
            "name": current_data.get("name", ""),
            "SASUrl": current_data.get("SASUrl", ""),
            "organizationIds": current_data.get("organizationIds", [0])
        }

        # Apply updates from the request
        for key, value in update_data.items():
            if key == "organizationId" and value is not None:
                # Convert organizationId to organizationIds array for API
                payload["organizationIds"] = [value]
            elif key in ["accountName", "accountKey", "containerName"] and value is not None:
                # If SDK fields are provided, rebuild SAS URL
                # Get current values first
                current_account_name = update_data.get("accountName", "azure-storage")
                current_container_name = update_data.get("containerName", "container")

                # Build new SAS URL with updated values
                sas_url = f"https://{current_account_name}.blob.core.windows.net/{current_container_name}?sv=2022-01-01&ss=b&srt=co&sp=rwdlacupx&se=2025-12-31T23:59:59Z&st=2024-01-01T00:00:00Z&spr=https&sig=updated"
                payload["SASUrl"] = sas_url
            elif key == "name" and value is not None:
                payload["name"] = value
            # Skip other SDK-specific fields that don't map to API

        response = self.http_client.put(
            f"evidences/repositories/azure-storage/{self.repository_id}",
            json_data=payload
        )

        if response.get("success"):
            repository_data = response.get("result", {})

            # Handle field mapping from API response back to SDK model
            if '_id' in repository_data:
                repository_data['id'] = repository_data['_id']
                del repository_data['_id']

            # Map organizationIds array back to organizationId for SDK model
            if 'organizationIds' in repository_data and repository_data['organizationIds']:
                repository_data['organizationId'] = repository_data['organizationIds'][0]

            # Extract Azure Storage fields from SASUrl for SDK model
            if 'SASUrl' in repository_data:
                sas_url = repository_data['SASUrl']
                # Parse accountName and containerName from SAS URL
                if sas_url and '://' in sas_url:
                    try:
                        # Extract account name from URL like https://accountname.blob.core.windows.net/container...
                        url_parts = sas_url.split('://', 1)[1].split('/')
                        if len(url_parts) >= 2:
                            domain_parts = url_parts[0].split('.')
                            if len(domain_parts) >= 1:
                                repository_data['accountName'] = domain_parts[0]
                            repository_data['containerName'] = url_parts[1].split('?')[0]
                    except (ValueError, IndexError, AttributeError):
                        # Fallback values if URL parsing fails
                        repository_data['accountName'] = 'azure-storage'
                        repository_data['containerName'] = 'container'

                # Provide a placeholder for accountKey since API doesn't return it
                repository_data['accountKey'] = 'hidden'

            return AzureStorageRepository(**repository_data)

        raise Exception(f"Failed to update Azure Storage repository: {response.get('error', 'Unknown error')}")





class ValidateAzureStorageRepositoryCommand(Command[ValidationResult]):
    """Command to validate Azure Storage repository."""

    def __init__(self, http_client: HTTPClient, request: ValidateRepositoryRequest):
        self.http_client = http_client
        self.request = request

    def execute(self) -> ValidationResult:
        """Execute the validate Azure Storage repository command."""
        payload = self.request.model_dump(exclude_none=True)

        # The validation API expects the same field structure as create API
        # Extract config if using the generic ValidateRepositoryRequest structure
        if 'config' in payload and isinstance(payload['config'], dict):
            payload = payload['config']

        # Apply the same field mapping as CreateAzureStorageRepositoryCommand
        # Handle field mapping for API compatibility
        if 'organizationId' in payload:
            # API expects organizationIds array, not organizationId
            if 'organizationIds' not in payload or not payload['organizationIds']:
                payload['organizationIds'] = [payload['organizationId']]
            del payload['organizationId']

        # Convert SDK Azure Storage fields to API SASUrl format
        if 'accountName' in payload and 'accountKey' in payload and 'containerName' in payload:
            # Build a basic SAS URL format from the individual components
            account_name = payload['accountName']
            container_name = payload['containerName']
            # Create a test SAS URL format (this would normally come from Azure)
            sas_url = f"https://{account_name}.blob.core.windows.net/{container_name}?sv=2022-01-01&ss=b&srt=co&sp=rwdlacupx&se=2025-12-31T23:59:59Z&st=2024-01-01T00:00:00Z&spr=https&sig=validate"
            payload['SASUrl'] = sas_url

            # Remove individual fields that API doesn't expect
            del payload['accountName']
            del payload['accountKey']
            del payload['containerName']

        # Remove SDK-specific fields that API doesn't expect
        if 'isDefault' in payload:
            del payload['isDefault']

        response = self.http_client.post("evidences/repositories/validate/azure-storage", json_data=payload)

        if response.get("success"):
            validation_data = response.get("result", {})
            return ValidationResult(**validation_data)

        raise Exception(f"Failed to validate Azure Storage repository: {response.get('error', 'Unknown error')}")


# FTPS Repository Commands

class CreateFTPSRepositoryCommand(Command[FTPSRepository]):
    """Command to create FTPS repository."""

    def __init__(self, http_client: HTTPClient, request: CreateFTPSRepositoryRequest):
        self.http_client = http_client
        self.request = request

    def execute(self) -> FTPSRepository:
        """Execute the create FTPS repository command."""
        payload = self.request.model_dump(exclude_none=True)

        # Handle field mapping for API compatibility
        if 'organizationId' in payload:
            # API expects organizationIds array, not organizationId
            if 'organizationIds' not in payload or not payload['organizationIds']:
                payload['organizationIds'] = [payload['organizationId']]
            del payload['organizationId']

        # Map remotePath to path for API
        if 'remotePath' in payload:
            payload['path'] = payload['remotePath']
            del payload['remotePath']

        # Remove SDK-specific fields that API doesn't expect
        if 'isDefault' in payload:
            del payload['isDefault']

        # Add required FTPS fields that might be missing
        if 'allowSelfSignedSSL' not in payload:
            payload['allowSelfSignedSSL'] = False

        if 'publicKey' not in payload:
            payload['publicKey'] = ""

        response = self.http_client.post("evidences/repositories/ftps", json_data=payload)

        if response.get("success"):
            repository_data = response.get("result", {})

            # Handle field mapping from API response back to SDK model
            if '_id' in repository_data:
                repository_data['id'] = repository_data['_id']
                del repository_data['_id']

            # Map organizationIds array back to organizationId for SDK model
            if 'organizationIds' in repository_data and repository_data['organizationIds']:
                repository_data['organizationId'] = repository_data['organizationIds'][0]

            # Map path back to remotePath for SDK model
            if 'path' in repository_data:
                repository_data['remotePath'] = repository_data['path']
                del repository_data['path']

            return FTPSRepository(**repository_data)

        raise Exception(f"Failed to create FTPS repository: {response.get('error', 'Unknown error')}")


class UpdateFTPSRepositoryCommand(Command[FTPSRepository]):
    """Command to update FTPS repository."""

    def __init__(self, http_client: HTTPClient, repository_id: str, request: UpdateFTPSRepositoryRequest):
        self.http_client = http_client
        self.repository_id = repository_id
        self.request = request

    def execute(self) -> FTPSRepository:
        """Execute the update FTPS repository command."""
        # The FTPS update API requires a complete payload, not partial updates
        # First, get the current repository data
        current_response = self.http_client.get(f"evidences/repositories/{self.repository_id}")

        if not current_response.get("success"):
            raise Exception(f"Failed to get current repository data: {current_response.get('error', 'Unknown error')}")

        current_data = current_response.get("result", {})

        # Create update payload by merging current data with updates
        update_data = self.request.model_dump()  # Don't exclude None to see all fields

        # Start with required fields from current repository
        payload = {
            "name": current_data.get("name", ""),
            "host": current_data.get("host", ""),
            "port": current_data.get("port", 21),
            "path": current_data.get("path", ""),
            "username": current_data.get("username", ""),
            "password": current_data.get("password", ""),  # Note: API may not return password
            "passive": current_data.get("passive", True),
            "allowSelfSignedSSL": current_data.get("allowSelfSignedSSL", False),
            "organizationIds": current_data.get("organizationIds", [0])
        }

        # Apply updates from the request
        for key, value in update_data.items():
            if key == "organizationId" and value is not None:
                # Convert organizationId to organizationIds array for API
                payload["organizationIds"] = [value]
            elif key == "remotePath" and value is not None:
                # Map remotePath to path for API
                payload["path"] = value
            elif value is not None:  # Only apply non-None updates
                payload[key] = value

        # Ensure required fields are always provided
        if not payload.get("password"):
            payload["password"] = "placeholder_password"  # API requires password but may not return it

        # For publicKey field, preserve from current data or use a minimal default if truly missing
        if "publicKey" not in payload:
            payload["publicKey"] = current_data.get("publicKey") or "default-public-key"

        response = self.http_client.put(
            f"evidences/repositories/ftps/{self.repository_id}",
            json_data=payload
        )

        if response.get("success"):
            repository_data = response.get("result", {})

            # Handle field mapping from API response back to SDK model
            if '_id' in repository_data:
                repository_data['id'] = repository_data['_id']
                del repository_data['_id']

            # Map organizationIds array back to organizationId for SDK model
            if 'organizationIds' in repository_data and repository_data['organizationIds']:
                repository_data['organizationId'] = repository_data['organizationIds'][0]

            # Map path back to remotePath for SDK model
            if 'path' in repository_data:
                repository_data['remotePath'] = repository_data['path']
                del repository_data['path']

            return FTPSRepository(**repository_data)

        raise Exception(f"Failed to update FTPS repository: {response.get('error', 'Unknown error')}")





class ValidateFTPSRepositoryCommand(Command[ValidationResult]):
    """Command to validate FTPS repository."""

    def __init__(self, http_client: HTTPClient, request: ValidateRepositoryRequest):
        self.http_client = http_client
        self.request = request

    def execute(self) -> ValidationResult:
        """Execute the validate FTPS repository command."""
        payload = self.request.model_dump(exclude_none=True)

        # The validation API expects the same field structure as create API
        # Extract config if using the generic ValidateRepositoryRequest structure
        if 'config' in payload and isinstance(payload['config'], dict):
            payload = payload['config']

        # Apply the same field mapping as CreateFTPSRepositoryCommand
        # Handle field mapping for API compatibility
        if 'organizationId' in payload:
            # API expects organizationIds array, not organizationId
            if 'organizationIds' not in payload or not payload['organizationIds']:
                payload['organizationIds'] = [payload['organizationId']]
            del payload['organizationId']

        # Map remotePath to path for API
        if 'remotePath' in payload:
            payload['path'] = payload['remotePath']
            del payload['remotePath']

        # Remove SDK-specific fields that API doesn't expect
        if 'isDefault' in payload:
            del payload['isDefault']

        # Add required FTPS fields that might be missing
        if 'allowSelfSignedSSL' not in payload:
            payload['allowSelfSignedSSL'] = False

        if 'publicKey' not in payload:
            payload['publicKey'] = ""

        response = self.http_client.post("evidences/repositories/validate/ftps", json_data=payload)

        if response.get("success"):
            validation_data = response.get("result", {})
            return ValidationResult(**validation_data)

        raise Exception(f"Failed to validate FTPS repository: {response.get('error', 'Unknown error')}")


# SFTP Repository Commands

class CreateSFTPRepositoryCommand(Command[SFTPRepository]):
    """Command to create SFTP repository."""

    def __init__(self, http_client: HTTPClient, request: CreateSFTPRepositoryRequest):
        self.http_client = http_client
        self.request = request

    def execute(self) -> SFTPRepository:
        """Execute the create SFTP repository command."""
        payload = self.request.model_dump(exclude_none=True)

        # Handle field mapping for API compatibility
        if 'organizationId' in payload:
            # API expects organizationIds array, not organizationId
            if 'organizationIds' not in payload or not payload['organizationIds']:
                payload['organizationIds'] = [payload['organizationId']]
            del payload['organizationId']

        # Map remotePath to path for API
        if 'remotePath' in payload:
            payload['path'] = payload['remotePath']
            del payload['remotePath']

        response = self.http_client.post("evidences/repositories/sftp", json_data=payload)

        if response.get("success"):
            repository_data = response.get("result", {})

            # Handle field mapping from API response back to SDK model
            if '_id' in repository_data:
                repository_data['id'] = repository_data['_id']
                del repository_data['_id']

            # Map organizationIds array back to organizationId for SDK model
            if 'organizationIds' in repository_data and repository_data['organizationIds']:
                repository_data['organizationId'] = repository_data['organizationIds'][0]

            # Map path back to remotePath for SDK model
            if 'path' in repository_data:
                repository_data['remotePath'] = repository_data['path']
                del repository_data['path']

            return SFTPRepository(**repository_data)

        raise Exception(f"Failed to create SFTP repository: {response.get('error', 'Unknown error')}")


class UpdateSFTPRepositoryCommand(Command[SFTPRepository]):
    """Command to update SFTP repository."""

    def __init__(self, http_client: HTTPClient, repository_id: str, request: UpdateSFTPRepositoryRequest):
        self.http_client = http_client
        self.repository_id = repository_id
        self.request = request

    def execute(self) -> SFTPRepository:
        """Execute the update SFTP repository command."""
        # The SFTP update API requires a complete payload, not partial updates
        # First, get the current repository data
        current_response = self.http_client.get(f"evidences/repositories/{self.repository_id}")

        if not current_response.get("success"):
            raise Exception(f"Failed to get current repository data: {current_response.get('error', 'Unknown error')}")

        current_data = current_response.get("result", {})

        # Create update payload by merging current data with updates
        update_data = self.request.model_dump()  # Don't exclude None to see all fields

        # Start with required fields from current repository
        payload = {
            "name": current_data.get("name", ""),
            "host": current_data.get("host", ""),
            "port": current_data.get("port", 22),
            "path": current_data.get("path", ""),
            "username": current_data.get("username", ""),
            "password": current_data.get("password", ""),  # Note: API may not return password
            "organizationIds": current_data.get("organizationIds", [0])
        }

        # Apply updates from the request
        for key, value in update_data.items():
            if key == "organizationId" and value is not None:
                # Convert organizationId to organizationIds array for API
                payload["organizationIds"] = [value]
            elif key == "remotePath" and value is not None:
                # Map remotePath to path for API
                payload["path"] = value
            elif value is not None:  # Only apply non-None updates
                payload[key] = value

        # Ensure password is provided (API requirement)
        if not payload.get("password"):
            payload["password"] = "placeholder_password"  # API requires password but may not return it

        response = self.http_client.put(
            f"evidences/repositories/sftp/{self.repository_id}",
            json_data=payload
        )

        if response.get("success"):
            repository_data = response.get("result", {})

            # Handle field mapping from API response back to SDK model
            if '_id' in repository_data:
                repository_data['id'] = repository_data['_id']
                del repository_data['_id']

            # Map organizationIds array back to organizationId for SDK model
            if 'organizationIds' in repository_data and repository_data['organizationIds']:
                repository_data['organizationId'] = repository_data['organizationIds'][0]

            # Map path back to remotePath for SDK model
            if 'path' in repository_data:
                repository_data['remotePath'] = repository_data['path']
                del repository_data['path']

            return SFTPRepository(**repository_data)

        raise Exception(f"Failed to update SFTP repository: {response.get('error', 'Unknown error')}")





# SMB Repository Commands

class CreateSMBRepositoryCommand(Command[SMBRepository]):
    """Command to create SMB repository."""

    def __init__(self, http_client: HTTPClient, request: CreateSMBRepositoryRequest):
        self.http_client = http_client
        self.request = request

    def execute(self) -> SMBRepository:
        """Execute the create SMB repository command."""
        payload = self.request.model_dump(exclude_none=True)

        # Handle field mapping for API compatibility
        if 'organizationId' in payload:
            # API expects organizationIds array, not organizationId
            if 'organizationIds' not in payload or not payload['organizationIds']:
                payload['organizationIds'] = [payload['organizationId']]
            del payload['organizationId']

        response = self.http_client.post("evidences/repositories/smb", json_data=payload)

        if response.get("success"):
            repository_data = response.get("result", {})

            # Handle field mapping from API response back to SDK model
            if '_id' in repository_data:
                repository_data['id'] = repository_data['_id']
                del repository_data['_id']

            # Map organizationIds array back to organizationId for SDK model
            if 'organizationIds' in repository_data and repository_data['organizationIds']:
                repository_data['organizationId'] = repository_data['organizationIds'][0]

            return SMBRepository(**repository_data)

        raise Exception(f"Failed to create SMB repository: {response.get('error', 'Unknown error')}")


class UpdateSMBRepositoryCommand(Command[SMBRepository]):
    """Command to update SMB repository."""

    def __init__(self, http_client: HTTPClient, repository_id: str, request: UpdateSMBRepositoryRequest):
        self.http_client = http_client
        self.repository_id = repository_id
        self.request = request

    def execute(self) -> SMBRepository:
        """Execute the update SMB repository command."""
        # The SMB update API requires a complete payload, not partial updates
        # First, get the current repository data
        current_response = self.http_client.get(f"evidences/repositories/{self.repository_id}")

        if not current_response.get("success"):
            raise Exception(f"Failed to get current repository data: {current_response.get('error', 'Unknown error')}")

        current_data = current_response.get("result", {})

        # Create update payload by merging current data with updates
        update_data = self.request.model_dump()  # Don't exclude None to see all fields

        # Start with required fields from current repository
        payload = {
            "name": current_data.get("name", ""),
            "path": current_data.get("path", ""),
            "username": current_data.get("username", ""),
            "password": current_data.get("password", ""),  # Note: API may not return password
            "organizationIds": current_data.get("organizationIds", [0])
        }

        # Apply updates from the request
        for key, value in update_data.items():
            if key == "organizationId" and value is not None:
                # Convert organizationId to organizationIds array for API
                payload["organizationIds"] = [value]
            elif value is not None:  # Only apply non-None updates
                payload[key] = value

        # Ensure password is provided (API requirement)
        if not payload.get("password"):
            payload["password"] = "placeholder_password"  # API requires password but may not return it

        response = self.http_client.put(
            f"evidences/repositories/smb/{self.repository_id}",
            json_data=payload
        )

        if response.get("success"):
            repository_data = response.get("result", {})

            # Handle field mapping from API response back to SDK model
            if '_id' in repository_data:
                repository_data['id'] = repository_data['_id']
                del repository_data['_id']

            # Map organizationIds array back to organizationId for SDK model
            if 'organizationIds' in repository_data and repository_data['organizationIds']:
                repository_data['organizationId'] = repository_data['organizationIds'][0]

            return SMBRepository(**repository_data)

        raise Exception(f"Failed to update SMB repository: {response.get('error', 'Unknown error')}")





"""Logger command classes for Binalyze AIR SDK."""

from typing import Optional, Dict, Any
from pathlib import Path
from ..models.logger import LogDownloadResponse
from ..queries.logger import DownloadLogsQuery


class LoggerCommand:
    """Base command class for logger operations."""
    
    def __init__(self, api_client):
        """Initialize logger command.
        
        Args:
            api_client: The Logger API client instance
        """
        self._api_client = api_client


class DownloadLogsCommand(LoggerCommand):
    """Command for downloading application logs as ZIP file.
    
    This command provides comprehensive log download functionality
    with options for latest logs only and automatic file saving.
    """
    
    def __init__(self, api_client, latest_log_file: bool = False):
        """Initialize download logs command.
        
        Args:
            api_client: The Logger API client instance
            latest_log_file: Whether to download only the latest log file
        """
        super().__init__(api_client)
        self._latest_log_file = latest_log_file

    def execute(self) -> LogDownloadResponse:
        """Execute download logs command.
        
        Returns:
            LogDownloadResponse with downloaded log data
            
        Raises:
            APIError: If the request fails
        """
        query = DownloadLogsQuery(self._latest_log_file)
        response_data, headers = self._api_client._download_logs_query(query)
        
        # Create response object from binary data and headers
        return LogDownloadResponse.from_response(response_data, headers)

    def download_and_save(self, save_path: Optional[str] = None) -> Dict[str, Any]:
        """Download logs and save to file.
        
        Args:
            save_path: Optional path where to save the logs.
                      If not provided, saves to current directory.
        
        Returns:
            Dictionary with download result and file information
        """
        try:
            # Download the logs
            log_response = self.execute()
            
            # Determine save path
            if save_path is None:
                save_path = log_response.filename or 'air_logs.zip'
            
            # Ensure directory exists
            Path(save_path).parent.mkdir(parents=True, exist_ok=True)
            
            # Save to file
            success = log_response.save_to_file(save_path)
            
            return {
                'success': success,
                'file_path': save_path,
                'file_size': log_response.size,
                'content_type': log_response.content_type,
                'filename': log_response.filename,
                'latest_only': self._latest_log_file
            }
            
        except Exception as e:
            return {
                'success': False,
                'error': str(e),
                'file_path': save_path,
                'latest_only': self._latest_log_file
            }

    def get_log_info(self) -> Dict[str, Any]:
        """Get information about available logs without downloading.
        
        Returns:
            Dictionary with log information
        """
        try:
            # Download logs to get metadata
            log_response = self.execute()
            
            return {
                'available': True,
                'size': log_response.size,
                'content_type': log_response.content_type,
                'filename': log_response.filename,
                'latest_only': self._latest_log_file
            }
            
        except Exception as e:
            return {
                'available': False,
                'error': str(e),
                'latest_only': self._latest_log_file
            }

    @property
    def latest_log_file(self) -> bool:
        """Get the latest log file flag.
        
        Returns:
            Whether downloading only the latest log file
        """
        return self._latest_log_file 


"""
API Tokens models for the Binalyze AIR SDK.
"""

from typing import Optional, List
from datetime import datetime
from pydantic import Field

from ..base import AIRBaseModel


class APIToken(AIRBaseModel):
    """API Token model."""
    
    id: str = Field(alias="_id")
    created_at: datetime = Field(alias="createdAt")
    updated_at: datetime = Field(alias="updatedAt")
    username: str
    name: str
    description: str
    token: Optional[str] = None  # Only returned on creation
    expiration_date: Optional[datetime] = Field(default=None, alias="expirationDate")
    last_used_at: Optional[datetime] = Field(default=None, alias="lastUsedAt")
    role_id: str = Field(alias="roleId")


class APITokensPaginatedResponse(AIRBaseModel):
    """Paginated response for API tokens."""
    
    entities: List[APIToken]
    filters: List[str]
    sortables: List[str]
    total_entity_count: int = Field(alias="totalEntityCount")
    current_page: int = Field(alias="currentPage")
    page_size: int = Field(alias="pageSize")
    previous_page: int = Field(alias="previousPage")
    total_page_count: int = Field(alias="totalPageCount")
    next_page: int = Field(alias="nextPage")


class CreateAPITokenRequest(AIRBaseModel):
    """Request model for creating API tokens."""
    
    name: str
    description: str = ""
    expiration_date: Optional[datetime] = Field(default=None, alias="expirationDate")
    role_id: str = Field(alias="roleId")


class UpdateAPITokenRequest(AIRBaseModel):
    """Request model for updating API tokens."""
    
    name: Optional[str] = Field(default=None)
    description: Optional[str] = Field(default=None)
    expiration_date: Optional[datetime] = Field(default=None, alias="expirationDate")
    role_id: Optional[str] = Field(default=None, alias="roleId")
        
    def model_dump(self, **kwargs):
        """Override model_dump to ensure proper serialization."""
        # Always exclude unset values for partial updates
        kwargs.setdefault('exclude_unset', True)
        kwargs.setdefault('by_alias', True)
        kwargs.setdefault('mode', 'json')
        return super().model_dump(**kwargs)


class APITokenFilter(AIRBaseModel):
    """Filter parameters for API tokens."""
    
    page_size: Optional[int] = Field(default=10, alias="pageSize")
    page_number: Optional[int] = Field(default=1, alias="pageNumber")
    sort_type: Optional[str] = Field(default="DESC", alias="sortType")  # ASC or DESC
    sort_by: Optional[str] = Field(default="createdAt", alias="sortBy")  # name, description, expirationDate, createdAt 


"""
Organization-related queries for the Binalyze AIR SDK.
"""

from typing import List, Optional

from ..base import Query
from ..models.organizations import (
    Organization, OrganizationUser, OrganizationRole, OrganizationLicense,
    OrganizationSettings, OrganizationFilter, OrganizationsPaginatedResponse,
    FilterOption, OrganizationUsersPaginatedResponse, CheckOrganizationNameExistsResponse,
    ShareableDeploymentInfoResponse
)
from ..http_client import HTTPClient


class ListOrganizationsQuery(Query[OrganizationsPaginatedResponse]):
    """Query to list organizations with optional filtering."""
    
    def __init__(self, http_client: HTTPClient, page: int = 1, page_size: int = 10, 
                 sort_by: str = "name", order: str = "asc", filter_params: Optional[OrganizationFilter] = None):
        self.http_client = http_client
        self.page = page
        self.page_size = page_size
        self.sort_by = sort_by
        self.order = order
        self.filter_params = filter_params or OrganizationFilter()
    
    def execute(self) -> OrganizationsPaginatedResponse:
        """Execute the query to list organizations with complete parameter support."""
        # Build query parameters
        params = {
            "pageNumber": self.page,
            "pageSize": self.page_size,
            "sortBy": self.sort_by,
            "sortType": self.order.upper(),  # ASC or DESC
        }
        
        # Add filter parameters using proper API structure
        filter_params = self.filter_params.to_params()
        params.update(filter_params)
        
        response = self.http_client.get("organizations", params=params)
        
        result = response.get("result", {})
        
        # Use Pydantic model_validate for automatic field mapping
        return OrganizationsPaginatedResponse.model_validate(result)


class GetOrganizationQuery(Query[Organization]):
    """Query to get a specific organization by ID."""
    
    def __init__(self, http_client: HTTPClient, organization_id: str):
        self.http_client = http_client
        self.organization_id = organization_id
    
    def execute(self) -> Organization:
        """Execute the query to get organization details."""
        response = self.http_client.get(f"organizations/{self.organization_id}")
        
        entity_data = response.get("result", {})
        
        # Use Pydantic model_validate for automatic field mapping via aliases
        return Organization.model_validate(entity_data)


class GetOrganizationUsersQuery(Query[OrganizationUsersPaginatedResponse]):
    """Query to get users for a specific organization with complete field mapping."""
    
    def __init__(self, http_client: HTTPClient, organization_id: str, page: int = 1, page_size: int = 10):
        self.http_client = http_client
        self.organization_id = organization_id
        self.page = page
        self.page_size = page_size
    
    def execute(self) -> OrganizationUsersPaginatedResponse:
        """Execute the query to get organization users with complete field mapping."""
        # Add pagination parameters
        params = {
            "pageNumber": self.page,
            "pageSize": self.page_size
        }
        
        response = self.http_client.get(f"organizations/{self.organization_id}/users", params=params)
        
        result = response.get("result", {})
        
        # Use Pydantic model_validate for automatic field mapping
        return OrganizationUsersPaginatedResponse.model_validate(result)


class CheckOrganizationNameExistsQuery(Query[CheckOrganizationNameExistsResponse]):
    """Query to check if an organization name exists."""
    
    def __init__(self, http_client: HTTPClient, name: str):
        self.http_client = http_client
        self.name = name
    
    def execute(self) -> CheckOrganizationNameExistsResponse:
        """Execute the query to check organization name availability."""
        params = {"name": self.name}
        
        response = self.http_client.get("organizations/check", params=params)
        
        # Use Pydantic model_validate for automatic field mapping
        return CheckOrganizationNameExistsResponse.model_validate(response)


class GetShareableDeploymentInfoQuery(Query[ShareableDeploymentInfoResponse]):
    """Query to get shareable deployment information by token."""
    
    def __init__(self, http_client: HTTPClient, token: str):
        self.http_client = http_client
        self.token = token
    
    def execute(self) -> ShareableDeploymentInfoResponse:
        """Execute the query to get shareable deployment information."""
        params = {"token": self.token}
        
        response = self.http_client.get("organizations/shareable-deployment", params=params)
        
        # Use Pydantic model_validate for automatic field mapping
        return ShareableDeploymentInfoResponse.model_validate(response)


class GetOrganizationRolesQuery(Query[List[OrganizationRole]]):
    """Query to get roles for a specific organization."""
    
    def __init__(self, http_client: HTTPClient, organization_id: str):
        self.http_client = http_client
        self.organization_id = organization_id
    
    def execute(self) -> List[OrganizationRole]:
        """Execute the query to get organization roles."""
        response = self.http_client.get(f"organizations/{self.organization_id}/roles")
        
        entities = response.get("result", {}).get("entities", [])
        
        roles = []
        for entity_data in entities:
            mapped_data = {
                "id": entity_data.get("_id"),
                "name": entity_data.get("name"),
                "description": entity_data.get("description"),
                "organization_id": entity_data.get("organizationId"),
                "permissions": entity_data.get("permissions", []),
                "created_at": entity_data.get("createdAt"),
                "updated_at": entity_data.get("updatedAt"),
                "created_by": entity_data.get("createdBy"),
                "is_system": entity_data.get("isSystem", False),
                "user_count": entity_data.get("userCount", 0),
            }
            
            # Remove None values
            mapped_data = {k: v for k, v in mapped_data.items() if v is not None}
            
            roles.append(OrganizationRole(**mapped_data))
        
        return roles


class GetOrganizationLicensesQuery(Query[List[OrganizationLicense]]):
    """Query to get licenses for a specific organization."""
    
    def __init__(self, http_client: HTTPClient, organization_id: str):
        self.http_client = http_client
        self.organization_id = organization_id
    
    def execute(self) -> List[OrganizationLicense]:
        """Execute the query to get organization licenses."""
        response = self.http_client.get(f"organizations/{self.organization_id}/licenses")
        
        entities = response.get("result", {}).get("entities", [])
        
        licenses = []
        for entity_data in entities:
            mapped_data = {
                "id": entity_data.get("_id"),
                "organization_id": entity_data.get("organizationId"),
                "license_type": entity_data.get("licenseType"),
                "total_licenses": entity_data.get("totalLicenses", 0),
                "used_licenses": entity_data.get("usedLicenses", 0),
                "valid_from": entity_data.get("validFrom"),
                "valid_until": entity_data.get("validUntil"),
                "features": entity_data.get("features", []),
                "is_active": entity_data.get("isActive", True),
            }
            
            # Remove None values
            mapped_data = {k: v for k, v in mapped_data.items() if v is not None}
            
            licenses.append(OrganizationLicense(**mapped_data))
        
        return licenses


class GetOrganizationSettingsQuery(Query[OrganizationSettings]):
    """Query to get settings for a specific organization."""
    
    def __init__(self, http_client: HTTPClient, organization_id: str):
        self.http_client = http_client
        self.organization_id = organization_id
    
    def execute(self) -> OrganizationSettings:
        """Execute the query to get organization settings."""
        response = self.http_client.get(f"organizations/{self.organization_id}/settings")
        
        entity_data = response.get("result", {})
        
        mapped_data = {
            "organization_id": entity_data.get("organizationId"),
            "retention_policy": entity_data.get("retentionPolicy", {}),
            "security_settings": entity_data.get("securitySettings", {}),
            "notification_settings": entity_data.get("notificationSettings", {}),
            "api_settings": entity_data.get("apiSettings", {}),
            "custom_settings": entity_data.get("customSettings", {}),
        }
        
        # Remove None values
        mapped_data = {k: v for k, v in mapped_data.items() if v is not None}
        
        return OrganizationSettings(**mapped_data) 